@@ -20,7 +20,6 @@
 #include "config.h"
 #include "wine/port.h"
 
-#define NONAMELESSUNION
 #include "d3d10core_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d10core);
@@ -153,8 +152,13 @@ static void STDMETHODCALLTYPE d3d10_device_VSSetShader(ID3D10Device *iface, ID3D
 static void STDMETHODCALLTYPE d3d10_device_DrawIndexed(ID3D10Device *iface,
         UINT index_count, UINT start_index_location, INT base_vertex_location)
 {
-    FIXME("iface %p, index_count %u, start_index_location %u, base_vertex_location %d stub!\n",
+    struct d3d10_device *This = (struct d3d10_device *)iface;
+
+    TRACE("iface %p, index_count %u, start_index_location %u, base_vertex_location %d.\n",
             iface, index_count, start_index_location, base_vertex_location);
+
+    IWineD3DDevice_SetBaseVertexIndex(This->wined3d_device, base_vertex_location);
+    IWineD3DDevice_DrawIndexedPrimitive(This->wined3d_device, start_index_location, index_count);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_Draw(ID3D10Device *iface,
@@ -182,7 +186,7 @@ static void STDMETHODCALLTYPE d3d10_device_IASetInputLayout(ID3D10Device *iface,
     TRACE("iface %p, input_layout %p\n", iface, input_layout);
 
     IWineD3DDevice_SetVertexDeclaration(This->wined3d_device,
-            ((struct d3d10_input_layout *)input_layout)->wined3d_decl);
+            input_layout ? ((struct d3d10_input_layout *)input_layout)->wined3d_decl : NULL);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_IASetVertexBuffers(ID3D10Device *iface,
@@ -198,15 +202,22 @@ static void STDMETHODCALLTYPE d3d10_device_IASetVertexBuffers(ID3D10Device *ifac
     for (i = 0; i < buffer_count; ++i)
     {
         IWineD3DDevice_SetStreamSource(This->wined3d_device, start_slot,
-                ((struct d3d10_buffer *)buffers[i])->wined3d_buffer, offsets[i], strides[i]);
+                buffers[i] ? ((struct d3d10_buffer *)buffers[i])->wined3d_buffer : NULL,
+                offsets[i], strides[i]);
     }
 }
 
 static void STDMETHODCALLTYPE d3d10_device_IASetIndexBuffer(ID3D10Device *iface,
         ID3D10Buffer *buffer, DXGI_FORMAT format, UINT offset)
 {
-    FIXME("iface %p, buffer %p, format %s, offset %u stub!\n",
+    struct d3d10_device *This = (struct d3d10_device *)iface;
+
+    TRACE("iface %p, buffer %p, format %s, offset %u.\n",
             iface, buffer, debug_dxgi_format(format), offset);
+
+    IWineD3DDevice_SetIndexBuffer(This->wined3d_device, buffer ? ((struct d3d10_buffer *)buffer)->wined3d_buffer : NULL,
+            wined3dformat_from_dxgi_format(format));
+    if (offset) FIXME("offset %u not supported.\n", offset);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_DrawIndexedInstanced(ID3D10Device *iface,
@@ -607,7 +618,6 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateBuffer(ID3D10Device *iface,
         const D3D10_BUFFER_DESC *desc, const D3D10_SUBRESOURCE_DATA *data, ID3D10Buffer **buffer)
 {
     struct d3d10_device *This = (struct d3d10_device *)iface;
-    struct wined3d_buffer_desc wined3d_desc;
     struct d3d10_buffer *object;
     HRESULT hr;
 
@@ -620,22 +630,10 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateBuffer(ID3D10Device *iface,
         return E_OUTOFMEMORY;
     }
 
-    object->vtbl = &d3d10_buffer_vtbl;
-    object->refcount = 1;
-
-    FIXME("Implement DXGI<->wined3d usage conversion\n");
-
-    wined3d_desc.byte_width = desc->ByteWidth;
-    wined3d_desc.usage = desc->Usage;
-    wined3d_desc.bind_flags = desc->BindFlags;
-    wined3d_desc.cpu_access_flags = desc->CPUAccessFlags;
-    wined3d_desc.misc_flags = desc->MiscFlags;
-
-    hr = IWineD3DDevice_CreateBuffer(This->wined3d_device, &wined3d_desc,
-            data ? data->pSysMem : NULL, (IUnknown *)object, &object->wined3d_buffer);
+    hr = d3d10_buffer_init(object, This, desc, data);
     if (FAILED(hr))
     {
-        ERR("CreateBuffer failed, returning %#x\n", hr);
+        WARN("Failed to initialize buffer, hr %#x.\n", hr);
         HeapFree(GetProcessHeap(), 0, object);
         return hr;
     }
@@ -671,46 +669,12 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture2D(ID3D10Device *ifac
         return E_OUTOFMEMORY;
     }
 
-    object->vtbl = &d3d10_texture2d_vtbl;
-    object->refcount = 1;
-    object->desc = *desc;
-
-    if (desc->MipLevels == 1 && desc->ArraySize == 1)
+    hr = d3d10_texture2d_init(object, This, desc);
+    if (FAILED(hr))
     {
-        IWineDXGIDevice *wine_device;
-
-        hr = ID3D10Device_QueryInterface(iface, &IID_IWineDXGIDevice, (void **)&wine_device);
-        if (FAILED(hr))
-        {
-            ERR("Device should implement IWineDXGIDevice\n");
-            HeapFree(GetProcessHeap(), 0, object);
-            return E_FAIL;
-        }
-
-        hr = IWineDXGIDevice_create_surface(wine_device, NULL, 0, NULL,
-                (IUnknown *)object, (void **)&object->dxgi_surface);
-        IWineDXGIDevice_Release(wine_device);
-        if (FAILED(hr))
-        {
-            ERR("Failed to create DXGI surface, returning %#x\n", hr);
-            HeapFree(GetProcessHeap(), 0, object);
-            return hr;
-        }
-
-        FIXME("Implement DXGI<->wined3d usage conversion\n");
-
-        hr = IWineD3DDevice_CreateSurface(This->wined3d_device, desc->Width, desc->Height,
-                wined3dformat_from_dxgi_format(desc->Format), FALSE, FALSE, 0,
-                &object->wined3d_surface, desc->Usage, WINED3DPOOL_DEFAULT,
-                desc->SampleDesc.Count > 1 ? desc->SampleDesc.Count : WINED3DMULTISAMPLE_NONE,
-                desc->SampleDesc.Quality, SURFACE_OPENGL, (IUnknown *)object);
-        if (FAILED(hr))
-        {
-            ERR("CreateSurface failed, returning %#x\n", hr);
-            IDXGISurface_Release(object->dxgi_surface);
-            HeapFree(GetProcessHeap(), 0, object);
-            return hr;
-        }
+        WARN("Failed to initialize texture, hr %#x.\n", hr);
+        HeapFree(GetProcessHeap(), 0, object);
+        return hr;
     }
 
     *texture = (ID3D10Texture2D *)object;
@@ -731,156 +695,37 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture3D(ID3D10Device *ifac
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateShaderResourceView(ID3D10Device *iface,
         ID3D10Resource *resource, const D3D10_SHADER_RESOURCE_VIEW_DESC *desc, ID3D10ShaderResourceView **view)
 {
-    FIXME("iface %p, resource %p, desc %p, view %p stub!\n", iface, resource, desc, view);
-
-    return E_NOTIMPL;
-}
-
-static IWineD3DResource *d3d10_device_wined3d_resource_from_resource(ID3D10Resource *resource)
-{
-    D3D10_RESOURCE_DIMENSION dimension;
-
-    ID3D10Resource_GetType(resource, &dimension);
-
-    switch(dimension)
-    {
-        case D3D10_RESOURCE_DIMENSION_BUFFER:
-            return (IWineD3DResource *)((struct d3d10_buffer *)resource)->wined3d_buffer;
-
-        case D3D10_RESOURCE_DIMENSION_TEXTURE2D:
-            return (IWineD3DResource *)((struct d3d10_texture2d *)resource)->wined3d_surface;
-
-        default:
-            FIXME("Unhandled resource dimension %#x\n", dimension);
-            return NULL;
-    }
-}
-
-static HRESULT d3d10_device_set_rtdesc_from_resource(D3D10_RENDER_TARGET_VIEW_DESC *desc, ID3D10Resource *resource)
-{
-    D3D10_RESOURCE_DIMENSION dimension;
+    struct d3d10_shader_resource_view *object;
     HRESULT hr;
 
-    ID3D10Resource_GetType(resource, &dimension);
+    TRACE("iface %p, resource %p, desc %p, view %p.\n", iface, resource, desc, view);
 
-    switch(dimension)
+    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
+    if (!object)
     {
-        case D3D10_RESOURCE_DIMENSION_TEXTURE1D:
-        {
-            ID3D10Texture1D *texture;
-            D3D10_TEXTURE1D_DESC texture_desc;
-
-            hr = ID3D10Resource_QueryInterface(resource, &IID_ID3D10Texture1D, (void **)&texture);
-            if (FAILED(hr))
-            {
-                ERR("Resource of type TEXTURE1D doesn't implement ID3D10Texture1D?\n");
-                return E_INVALIDARG;
-            }
-
-            ID3D10Texture1D_GetDesc(texture, &texture_desc);
-            ID3D10Texture1D_Release(texture);
-
-            desc->Format = texture_desc.Format;
-            if (texture_desc.ArraySize == 1)
-            {
-                desc->ViewDimension = D3D10_RTV_DIMENSION_TEXTURE1D;
-                desc->u.Texture1D.MipSlice = 0;
-            }
-            else
-            {
-                desc->ViewDimension = D3D10_RTV_DIMENSION_TEXTURE1DARRAY;
-                desc->u.Texture1DArray.MipSlice = 0;
-                desc->u.Texture1DArray.FirstArraySlice = 0;
-                desc->u.Texture1DArray.ArraySize = 1;
-            }
-
-            return S_OK;
-        }
-
-        case D3D10_RESOURCE_DIMENSION_TEXTURE2D:
-        {
-            ID3D10Texture2D *texture;
-            D3D10_TEXTURE2D_DESC texture_desc;
-
-            hr = ID3D10Resource_QueryInterface(resource, &IID_ID3D10Texture2D, (void **)&texture);
-            if (FAILED(hr))
-            {
-                ERR("Resource of type TEXTURE2D doesn't implement ID3D10Texture2D?\n");
-                return E_INVALIDARG;
-            }
-
-            ID3D10Texture2D_GetDesc(texture, &texture_desc);
-            ID3D10Texture2D_Release(texture);
-
-            desc->Format = texture_desc.Format;
-            if (texture_desc.ArraySize == 1)
-            {
-                if (texture_desc.SampleDesc.Count == 1)
-                {
-                    desc->ViewDimension = D3D10_RTV_DIMENSION_TEXTURE2D;
-                    desc->u.Texture2D.MipSlice = 0;
-                }
-                else
-                {
-                    desc->ViewDimension = D3D10_RTV_DIMENSION_TEXTURE2DMS;
-                }
-            }
-            else
-            {
-                if (texture_desc.SampleDesc.Count == 1)
-                {
-                    desc->ViewDimension = D3D10_RTV_DIMENSION_TEXTURE2DARRAY;
-                    desc->u.Texture2DArray.MipSlice = 0;
-                    desc->u.Texture2DArray.FirstArraySlice = 0;
-                    desc->u.Texture2DArray.ArraySize = 1;
-                }
-                else
-                {
-                    desc->ViewDimension = D3D10_RTV_DIMENSION_TEXTURE2DMSARRAY;
-                    desc->u.Texture2DMSArray.FirstArraySlice = 0;
-                    desc->u.Texture2DMSArray.ArraySize = 1;
-                }
-            }
-
-            return S_OK;
-        }
-
-        case D3D10_RESOURCE_DIMENSION_TEXTURE3D:
-        {
-            ID3D10Texture3D *texture;
-            D3D10_TEXTURE3D_DESC texture_desc;
-
-            hr = ID3D10Resource_QueryInterface(resource, &IID_ID3D10Texture3D, (void **)&texture);
-            if (FAILED(hr))
-            {
-                ERR("Resource of type TEXTURE3D doesn't implement ID3D10Texture3D?\n");
-                return E_INVALIDARG;
-            }
-
-            ID3D10Texture3D_GetDesc(texture, &texture_desc);
-            ID3D10Texture3D_Release(texture);
-
-            desc->Format = texture_desc.Format;
-            desc->ViewDimension = D3D10_RTV_DIMENSION_TEXTURE3D;
-            desc->u.Texture3D.MipSlice = 0;
-            desc->u.Texture3D.FirstWSlice = 0;
-            desc->u.Texture3D.WSize = 1;
-
-            return S_OK;
-        }
-
-        default:
-            FIXME("Unhandled resource dimension %#x\n", dimension);
-            return E_INVALIDARG;
+        ERR("Failed to allocate D3D10 shader resource view object memory.\n");
+        return E_OUTOFMEMORY;
     }
+
+    hr = d3d10_shader_resource_view_init(object);
+    if (FAILED(hr))
+    {
+        WARN("Failed to initialize shader resource view, hr %#x.\n", hr);
+        HeapFree(GetProcessHeap(), 0, object);
+        return hr;
+    }
+
+    TRACE("Created shader resource view %p.\n", object);
+    *view = (ID3D10ShaderResourceView *)object;
+
+    return S_OK;
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateRenderTargetView(ID3D10Device *iface,
         ID3D10Resource *resource, const D3D10_RENDER_TARGET_VIEW_DESC *desc, ID3D10RenderTargetView **view)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
     struct d3d10_rendertarget_view *object;
-    IWineD3DResource *wined3d_resource;
+    HRESULT hr;
 
     TRACE("iface %p, resource %p, desc %p, view %p stub!\n", iface, resource, desc, view);
 
@@ -891,33 +736,15 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateRenderTargetView(ID3D10Devic
         return E_OUTOFMEMORY;
     }
 
-    object->vtbl = &d3d10_rendertarget_view_vtbl;
-    object->refcount = 1;
-
-    if (!desc)
+    hr = d3d10_rendertarget_view_init(object, (struct d3d10_device *)iface, resource, desc);
+    if (FAILED(hr))
     {
-        HRESULT hr = d3d10_device_set_rtdesc_from_resource(&object->desc, resource);
-        if (FAILED(hr))
-        {
-            HeapFree(GetProcessHeap(), 0, object);
-            return hr;
-        }
-    }
-    else
-    {
-        object->desc = *desc;
-    }
-
-    wined3d_resource = d3d10_device_wined3d_resource_from_resource(resource);
-    if (!wined3d_resource)
-    {
-        FIXME("Failed to get wined3d resource for d3d10 resource %p\n", resource);
+        WARN("Failed to initialize rendertarget view, hr %#x.\n", hr);
         HeapFree(GetProcessHeap(), 0, object);
-        return E_FAIL;
+        return hr;
     }
-    IWineD3DDevice_CreateRendertargetView(This->wined3d_device,
-            wined3d_resource, (IUnknown *)object, &object->wined3d_view);
 
+    TRACE("Created rendertarget view %p.\n", object);
     *view = (ID3D10RenderTargetView *)object;
 
     return S_OK;
@@ -926,9 +753,30 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateRenderTargetView(ID3D10Devic
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateDepthStencilView(ID3D10Device *iface,
         ID3D10Resource *resource, const D3D10_DEPTH_STENCIL_VIEW_DESC *desc, ID3D10DepthStencilView **view)
 {
-    FIXME("iface %p, resource %p, desc %p, view %p stub!\n", iface, resource, desc, view);
+    struct d3d10_depthstencil_view *object;
+    HRESULT hr;
 
-    return E_NOTIMPL;
+    TRACE("iface %p, resource %p, desc %p, view %p.\n", iface, resource, desc, view);
+
+    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
+    if (!object)
+    {
+        ERR("Failed to allocate D3D10 depthstencil view object memory.\n");
+        return E_OUTOFMEMORY;
+    }
+
+    hr = d3d10_depthstencil_view_init(object);
+    if (FAILED(hr))
+    {
+        WARN("Failed to initialize depthstencil view, hr %#x.\n", hr);
+        HeapFree(GetProcessHeap(), 0, object);
+        return hr;
+    }
+
+    TRACE("Created depthstencil view %p.\n", object);
+    *view = (ID3D10DepthStencilView *)object;
+
+    return S_OK;
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateInputLayout(ID3D10Device *iface,
@@ -937,8 +785,6 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateInputLayout(ID3D10Device *if
 {
     struct d3d10_device *This = (struct d3d10_device *)iface;
     struct d3d10_input_layout *object;
-    WINED3DVERTEXELEMENT *wined3d_elements;
-    UINT wined3d_element_count;
     HRESULT hr;
 
     TRACE("iface %p, element_descs %p, element_count %u, shader_byte_code %p,"
@@ -953,22 +799,16 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateInputLayout(ID3D10Device *if
         return E_OUTOFMEMORY;
     }
 
-    object->vtbl = &d3d10_input_layout_vtbl;
-    object->refcount = 1;
-
-    hr = d3d10_input_layout_to_wined3d_declaration(element_descs, element_count,
-            shader_byte_code, shader_byte_code_length, &wined3d_elements, &wined3d_element_count);
+    hr = d3d10_input_layout_init(object, This, element_descs, element_count,
+            shader_byte_code, shader_byte_code_length);
     if (FAILED(hr))
     {
+        WARN("Failed to initialize input layout, hr %#x.\n", hr);
         HeapFree(GetProcessHeap(), 0, object);
         return hr;
     }
 
-    IWineD3DDevice_CreateVertexDeclaration(This->wined3d_device, &object->wined3d_decl,
-            (IUnknown *)object, wined3d_elements, wined3d_element_count);
-
-    HeapFree(GetProcessHeap(), 0, wined3d_elements);
-
+    TRACE("Created input layout %p.\n", object);
     *input_layout = (ID3D10InputLayout *)object;
 
     return S_OK;
@@ -979,7 +819,6 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateVertexShader(ID3D10Device *i
 {
     struct d3d10_device *This = (struct d3d10_device *)iface;
     struct d3d10_vertex_shader *object;
-    struct d3d10_shader_info shader_info;
     HRESULT hr;
 
     TRACE("iface %p, byte_code %p, byte_code_length %lu, shader %p\n",
@@ -992,29 +831,15 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateVertexShader(ID3D10Device *i
         return E_OUTOFMEMORY;
     }
 
-    object->vtbl = &d3d10_vertex_shader_vtbl;
-    object->refcount = 1;
-
-    shader_info.output_signature = &object->output_signature;
-    hr = shader_extract_from_dxbc(byte_code, byte_code_length, &shader_info);
+    hr = d3d10_vertex_shader_init(object, This, byte_code, byte_code_length);
     if (FAILED(hr))
     {
-        ERR("Failed to extract shader, hr %#x\n", hr);
-        HeapFree(GetProcessHeap(), 0, object);
-        return hr;
-    }
-
-    hr = IWineD3DDevice_CreateVertexShader(This->wined3d_device,
-            shader_info.shader_code, &object->output_signature,
-            &object->wined3d_shader, (IUnknown *)object);
-    if (FAILED(hr))
-    {
-        ERR("CreateVertexShader failed, hr %#x\n", hr);
-        shader_free_signature(&object->output_signature);
+        WARN("Failed to initialize vertex shader, hr %#x.\n", hr);
         HeapFree(GetProcessHeap(), 0, object);
         return hr;
     }
 
+    TRACE("Created vertex shader %p.\n", object);
     *shader = (ID3D10VertexShader *)object;
 
     return S_OK;
@@ -1024,6 +849,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateGeometryShader(ID3D10Device
         const void *byte_code, SIZE_T byte_code_length, ID3D10GeometryShader **shader)
 {
     struct d3d10_geometry_shader *object;
+    HRESULT hr;
 
     FIXME("iface %p, byte_code %p, byte_code_length %lu, shader %p stub!\n",
             iface, byte_code, byte_code_length, shader);
@@ -1035,9 +861,14 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateGeometryShader(ID3D10Device
         return E_OUTOFMEMORY;
     }
 
-    object->vtbl = &d3d10_geometry_shader_vtbl;
-    object->refcount = 1;
+    hr = d3d10_geometry_shader_init(object);
+    if (FAILED(hr))
+    {
+        WARN("Failed to initialize geometry shader, hr %#x.\n", hr);
+        HeapFree(GetProcessHeap(), 0, object);
+    }
 
+    TRACE("Created geometry shader %p.\n", object);
     *shader = (ID3D10GeometryShader *)object;
 
     return S_OK;
@@ -1060,7 +891,6 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreatePixelShader(ID3D10Device *if
 {
     struct d3d10_device *This = (struct d3d10_device *)iface;
     struct d3d10_pixel_shader *object;
-    struct d3d10_shader_info shader_info;
     HRESULT hr;
 
     TRACE("iface %p, byte_code %p, byte_code_length %lu, shader %p\n",
@@ -1073,29 +903,15 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreatePixelShader(ID3D10Device *if
         return E_OUTOFMEMORY;
     }
 
-    object->vtbl = &d3d10_pixel_shader_vtbl;
-    object->refcount = 1;
-
-    shader_info.output_signature = &object->output_signature;
-    hr = shader_extract_from_dxbc(byte_code, byte_code_length, &shader_info);
+    hr = d3d10_pixel_shader_init(object, This, byte_code, byte_code_length);
     if (FAILED(hr))
     {
-        ERR("Failed to extract shader, hr %#x\n", hr);
-        HeapFree(GetProcessHeap(), 0, object);
-        return hr;
-    }
-
-    hr = IWineD3DDevice_CreatePixelShader(This->wined3d_device,
-            shader_info.shader_code, &object->output_signature,
-            &object->wined3d_shader, (IUnknown *)object);
-    if (FAILED(hr))
-    {
-        ERR("CreatePixelShader failed, hr %#x\n", hr);
-        shader_free_signature(&object->output_signature);
+        WARN("Failed to initialize pixel shader, hr %#x.\n", hr);
         HeapFree(GetProcessHeap(), 0, object);
         return hr;
     }
 
+    TRACE("Created pixel shader %p.\n", object);
     *shader = (ID3D10PixelShader *)object;
 
     return S_OK;
@@ -1104,33 +920,117 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreatePixelShader(ID3D10Device *if
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateBlendState(ID3D10Device *iface,
         const D3D10_BLEND_DESC *desc, ID3D10BlendState **blend_state)
 {
-    FIXME("iface %p, desc %p, blend_state %p stub!\n", iface, desc, blend_state);
+    struct d3d10_blend_state *object;
+    HRESULT hr;
 
-    return E_NOTIMPL;
+    TRACE("iface %p, desc %p, blend_state %p.\n", iface, desc, blend_state);
+
+    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
+    if (!object)
+    {
+        ERR("Failed to allocate D3D10 blend state object memory.\n");
+        return E_OUTOFMEMORY;
+    }
+
+    hr = d3d10_blend_state_init(object);
+    if (FAILED(hr))
+    {
+        WARN("Failed to initialize blend state, hr %#x.\n", hr);
+        HeapFree(GetProcessHeap(), 0, object);
+        return hr;
+    }
+
+    TRACE("Created blend state %p.\n", object);
+    *blend_state = (ID3D10BlendState *)object;
+
+    return S_OK;
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateDepthStencilState(ID3D10Device *iface,
         const D3D10_DEPTH_STENCIL_DESC *desc, ID3D10DepthStencilState **depth_stencil_state)
 {
-    FIXME("iface %p, desc %p, depth_stencil_state %p stub!\n", iface, desc, depth_stencil_state);
+    struct d3d10_depthstencil_state *object;
+    HRESULT hr;
 
-    return E_NOTIMPL;
+    TRACE("iface %p, desc %p, depth_stencil_state %p.\n", iface, desc, depth_stencil_state);
+
+    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
+    if (!object)
+    {
+        ERR("Failed to allocate D3D10 depthstencil state object memory.\n");
+        return E_OUTOFMEMORY;
+    }
+
+    hr = d3d10_depthstencil_state_init(object);
+    if (FAILED(hr))
+    {
+        WARN("Failed to initialize depthstencil state, hr %#x.\n", hr);
+        HeapFree(GetProcessHeap(), 0, object);
+        return hr;
+    }
+
+    TRACE("Created depthstencil state %p.\n", object);
+    *depth_stencil_state = (ID3D10DepthStencilState *)object;
+
+    return S_OK;
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateRasterizerState(ID3D10Device *iface,
         const D3D10_RASTERIZER_DESC *desc, ID3D10RasterizerState **rasterizer_state)
 {
-    FIXME("iface %p, desc %p, rasterizer_state %p stub!\n", iface, desc, rasterizer_state);
+    struct d3d10_rasterizer_state *object;
+    HRESULT hr;
 
-    return E_NOTIMPL;
+    TRACE("iface %p, desc %p, rasterizer_state %p.\n", iface, desc, rasterizer_state);
+
+    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
+    if (!object)
+    {
+        ERR("Failed to allocate D3D10 rasterizer state object memory.\n");
+        return E_OUTOFMEMORY;
+    }
+
+    hr = d3d10_rasterizer_state_init(object);
+    if (FAILED(hr))
+    {
+        WARN("Failed to initialize rasterizer state, hr %#x.\n", hr);
+        HeapFree(GetProcessHeap(), 0, object);
+        return hr;
+    }
+
+    TRACE("Created rasterizer state %p.\n", object);
+    *rasterizer_state = (ID3D10RasterizerState *)object;
+
+    return S_OK;
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateSamplerState(ID3D10Device *iface,
         const D3D10_SAMPLER_DESC *desc, ID3D10SamplerState **sampler_state)
 {
-    FIXME("iface %p, desc %p, sampler_state %p stub!\n", iface, desc, sampler_state);
+    struct d3d10_sampler_state *object;
+    HRESULT hr;
 
-    return E_NOTIMPL;
+    FIXME("iface %p, desc %p, sampler_state %p.\n", iface, desc, sampler_state);
+
+    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
+    if (!object)
+    {
+        ERR("Failed to allocate D3D10 sampler state object memory.\n");
+        return E_OUTOFMEMORY;
+    }
+
+    hr = d3d10_sampler_state_init(object);
+    if (FAILED(hr))
+    {
+        WARN("Failed to initialize sampler state, hr %#x.\n", hr);
+        HeapFree(GetProcessHeap(), 0, object);
+        return hr;
+    }
+
+    TRACE("Created sampler state %p.\n", object);
+    *sampler_state = (ID3D10SamplerState *)object;
+
+    return S_OK;
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateQuery(ID3D10Device *iface,
@@ -1218,7 +1118,7 @@ static void STDMETHODCALLTYPE d3d10_device_GetTextFilterSize(ID3D10Device *iface
     FIXME("iface %p, width %p, height %p stub!\n", iface, width, height);
 }
 
-const struct ID3D10DeviceVtbl d3d10_device_vtbl =
+static const struct ID3D10DeviceVtbl d3d10_device_vtbl =
 {
     /* IUnknown methods */
     d3d10_device_QueryInterface,
@@ -1322,7 +1222,7 @@ const struct ID3D10DeviceVtbl d3d10_device_vtbl =
     d3d10_device_GetTextFilterSize,
 };
 
-const struct IUnknownVtbl d3d10_device_inner_unknown_vtbl =
+static const struct IUnknownVtbl d3d10_device_inner_unknown_vtbl =
 {
     /* IUnknown methods */
     d3d10_device_inner_QueryInterface,
@@ -1402,6 +1302,8 @@ static HRESULT STDMETHODCALLTYPE device_parent_CreateSurface(IWineD3DDeviceParen
     }
 
     *surface = texture->wined3d_surface;
+    IWineD3DSurface_AddRef(*surface);
+    ID3D10Texture2D_Release((ID3D10Texture2D *)texture);
 
     return S_OK;
 }
@@ -1441,6 +1343,8 @@ static HRESULT STDMETHODCALLTYPE device_parent_CreateRenderTarget(IWineD3DDevice
     }
 
     *surface = texture->wined3d_surface;
+    IWineD3DSurface_AddRef(*surface);
+    ID3D10Texture2D_Release((ID3D10Texture2D *)texture);
 
     return S_OK;
 }
@@ -1480,6 +1384,8 @@ static HRESULT STDMETHODCALLTYPE device_parent_CreateDepthStencilSurface(IWineD3
     }
 
     *surface = texture->wined3d_surface;
+    IWineD3DSurface_AddRef(*surface);
+    ID3D10Texture2D_Release((ID3D10Texture2D *)texture);
 
     return S_OK;
 }
@@ -1520,7 +1426,7 @@ static HRESULT STDMETHODCALLTYPE device_parent_CreateSwapChain(IWineD3DDevicePar
     return S_OK;
 }
 
-const struct IWineD3DDeviceParentVtbl d3d10_wined3d_device_parent_vtbl =
+static const struct IWineD3DDeviceParentVtbl d3d10_wined3d_device_parent_vtbl =
 {
     /* IUnknown methods */
     device_parent_QueryInterface,
@@ -1534,3 +1440,12 @@ const struct IWineD3DDeviceParentVtbl d3d10_wined3d_device_parent_vtbl =
     device_parent_CreateVolume,
     device_parent_CreateSwapChain,
 };
+
+void d3d10_device_init(struct d3d10_device *device, void *outer_unknown)
+{
+    device->vtbl = &d3d10_device_vtbl;
+    device->inner_unknown_vtbl = &d3d10_device_inner_unknown_vtbl;
+    device->device_parent_vtbl = &d3d10_wined3d_device_parent_vtbl;
+    device->refcount = 1;
+    device->outer_unknown = outer_unknown;
+}
