@@ -24,6 +24,7 @@
 #include "winbase.h"
 #include "winuser.h"
 #include "ole2.h"
+#include "mshtmdid.h"
 
 #include "mshtml_private.h"
 #include "htmlevent.h"
@@ -118,6 +119,7 @@ typedef struct {
     LPCWSTR name;
     LPCWSTR attr_name;
     DWORD type;
+    DISPID dispid;
     DWORD flags;
 } event_info_t;
 
@@ -126,24 +128,42 @@ typedef struct {
 #define EVENT_FORWARDBODY        0x0004
 
 static const event_info_t event_info[] = {
-    {beforeunloadW,      onbeforeunloadW,      EVENTT_NONE,  EVENT_DEFAULTLISTENER|EVENT_FORWARDBODY},
-    {blurW,              onblurW,              EVENTT_HTML,  EVENT_DEFAULTLISTENER},
-    {changeW,            onchangeW,            EVENTT_HTML,  EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
-    {clickW,             onclickW,             EVENTT_MOUSE, EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
-    {dblclickW,          ondblclickW,          EVENTT_MOUSE, EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
-    {dragW,              ondragW,              EVENTT_MOUSE, 0},
-    {dragstartW,         ondragstartW,         EVENTT_MOUSE, 0},
-    {focusW,             onfocusW,             EVENTT_HTML,  EVENT_DEFAULTLISTENER},
-    {keydownW,           onkeydownW,           EVENTT_KEY,   EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
-    {keyupW,             onkeyupW,             EVENTT_KEY,   EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
-    {loadW,              onloadW,              EVENTT_HTML,  0},
-    {mousedownW,         onmousedownW,         EVENTT_MOUSE, EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
-    {mouseoutW,          onmouseoutW,          EVENTT_MOUSE, EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
-    {mouseoverW,         onmouseoverW,         EVENTT_MOUSE, EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
-    {mouseupW,           onmouseupW,           EVENTT_MOUSE, EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
-    {pasteW,             onpasteW,             EVENTT_NONE,  0},
-    {readystatechangeW,  onreadystatechangeW,  EVENTT_NONE,  0},
-    {selectstartW,       onselectstartW,       EVENTT_MOUSE, 0}
+    {beforeunloadW,      onbeforeunloadW,      EVENTT_NONE,   DISPID_EVMETH_ONBEFOREUNLOAD,
+        EVENT_DEFAULTLISTENER|EVENT_FORWARDBODY},
+    {blurW,              onblurW,              EVENTT_HTML,   DISPID_EVMETH_ONBLUR,
+        EVENT_DEFAULTLISTENER},
+    {changeW,            onchangeW,            EVENTT_HTML,   DISPID_EVMETH_ONCHANGE,
+        EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
+    {clickW,             onclickW,             EVENTT_MOUSE,  DISPID_EVMETH_ONCLICK,
+        EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
+    {dblclickW,          ondblclickW,          EVENTT_MOUSE,  DISPID_EVMETH_ONDBLCLICK,
+        EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
+    {dragW,              ondragW,              EVENTT_MOUSE,  DISPID_EVMETH_ONDRAG,
+        0},
+    {dragstartW,         ondragstartW,         EVENTT_MOUSE,  DISPID_EVMETH_ONDRAGSTART,
+        0},
+    {focusW,             onfocusW,             EVENTT_HTML,   DISPID_EVMETH_ONFOCUS,
+        EVENT_DEFAULTLISTENER},
+    {keydownW,           onkeydownW,           EVENTT_KEY,    DISPID_EVMETH_ONKEYDOWN,
+        EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
+    {keyupW,             onkeyupW,             EVENTT_KEY,    DISPID_EVMETH_ONKEYUP,
+        EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
+    {loadW,              onloadW,              EVENTT_HTML,   DISPID_EVMETH_ONLOAD,
+        0},
+    {mousedownW,         onmousedownW,         EVENTT_MOUSE,  DISPID_EVMETH_ONMOUSEDOWN,
+        EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
+    {mouseoutW,          onmouseoutW,          EVENTT_MOUSE,  DISPID_EVMETH_ONMOUSEOUT,
+        EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
+    {mouseoverW,         onmouseoverW,         EVENTT_MOUSE,  DISPID_EVMETH_ONMOUSEOVER,
+        EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
+    {mouseupW,           onmouseupW,           EVENTT_MOUSE,  DISPID_EVMETH_ONMOUSEUP,
+        EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
+    {pasteW,             onpasteW,             EVENTT_NONE,   DISPID_EVMETH_ONPASTE,
+        0},
+    {readystatechangeW,  onreadystatechangeW,  EVENTT_NONE,   DISPID_EVMETH_ONREADYSTATECHANGE,
+        0},
+    {selectstartW,       onselectstartW,       EVENTT_MOUSE,  DISPID_EVMETH_ONSELECTSTART,
+        0}
 };
 
 eventid_t str_to_eid(LPCWSTR str)
@@ -738,17 +758,68 @@ static IHTMLEventObj *create_event(HTMLDOMNode *target, eventid_t eid, nsIDOMEve
     return HTMLEVENTOBJ(ret);
 }
 
-static void call_event_handlers(HTMLDocumentNode *doc, IHTMLEventObj *event_obj, event_target_t *event_target,
-        eventid_t eid, IDispatch *this_obj)
+static HRESULT call_cp_func(IDispatch *disp, DISPID dispid)
 {
-    handler_vector_t *handler_vector;
+    DISPPARAMS dp = {NULL,NULL,0,0};
+    ULONG argerr;
+    EXCEPINFO ei;
+    VARIANT vres;
+    HRESULT hres;
+
+    V_VT(&vres) = VT_EMPTY;
+    memset(&ei, 0, sizeof(ei));
+    hres = IDispatch_Invoke(disp, dispid, &IID_NULL, 0, DISPATCH_METHOD, &dp, &vres, &ei, &argerr);
+    if(SUCCEEDED(hres) && V_VT(&vres) != VT_EMPTY) {
+        FIXME("handle result %s\n", debugstr_variant(&vres));
+        VariantClear(&vres);
+    }
+
+    return hres;
+}
+
+static BOOL is_cp_event(ConnectionPoint *cp, DISPID dispid)
+{
+    cp_static_data_t *data;
+    int min, max, i;
+    HRESULT hres;
+
+    data = cp->data;
+    if(!data)
+        return FALSE;
+
+    if(!data->ids) {
+        hres = get_dispids(data->tid, &data->id_cnt, &data->ids);
+        if(FAILED(hres))
+            return FALSE;
+    }
+
+    min = 0;
+    max = data->id_cnt-1;
+    while(min <= max) {
+        i = (min+max)/2;
+        if(data->ids[i] == dispid)
+            return TRUE;
+
+        if(data->ids[i] < dispid)
+            min = i+1;
+        else
+            max = i-1;
+    }
+
+    return FALSE;
+}
+
+static void call_event_handlers(HTMLDocumentNode *doc, IHTMLEventObj *event_obj, event_target_t *event_target,
+        ConnectionPointContainer *cp_container, eventid_t eid, IDispatch *this_obj)
+{
+    handler_vector_t *handler_vector = NULL;
     DWORD i;
     HRESULT hres;
 
-    if(!event_target || !(handler_vector = event_target->event_table[eid]))
-        return;
+    if(event_target)
+        handler_vector = event_target->event_table[eid];
 
-    if(handler_vector->handler_prop) {
+    if(handler_vector && handler_vector->handler_prop) {
         DISPID named_arg = DISPID_THIS;
         VARIANTARG arg;
         DISPPARAMS dp = {&arg, &named_arg, 1, 1};
@@ -764,7 +835,7 @@ static void call_event_handlers(HTMLDocumentNode *doc, IHTMLEventObj *event_obj,
             WARN("%s <<< %08x\n", debugstr_w(event_info[eid].name), hres);
     }
 
-    if(handler_vector->handler_cnt) {
+    if(handler_vector && handler_vector->handler_cnt) {
         VARIANTARG arg;
         DISPPARAMS dp = {&arg, NULL, 1, 0};
 
@@ -782,6 +853,26 @@ static void call_event_handlers(HTMLDocumentNode *doc, IHTMLEventObj *event_obj,
             }
         }
     }
+
+    if(cp_container) {
+        ConnectionPoint *cp;
+
+        if(cp_container->forward_container)
+            cp_container = cp_container->forward_container;
+
+        for(cp = cp_container->cp_list; cp; cp = cp->next) {
+            if(cp->sinks_size && is_cp_event(cp, event_info[eid].dispid)) {
+                for(i=0; i < cp->sinks_size; i++) {
+                    TRACE("cp %s [%d] >>>\n", debugstr_w(event_info[eid].name), i);
+                    hres = call_cp_func(cp->sinks[i].disp, event_info[eid].dispid);
+                    if(hres == S_OK)
+                        TRACE("cp %s [%d] <<<\n", debugstr_w(event_info[eid].name), i);
+                    else
+                        WARN("cp %s [%d] <<< %08x\n", debugstr_w(event_info[eid].name), i, hres);
+                }
+            }
+        }
+    }
 }
 
 void fire_event(HTMLDocumentNode *doc, eventid_t eid, nsIDOMNode *target, nsIDOMEvent *nsevent)
@@ -805,7 +896,8 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, nsIDOMNode *target, nsIDOM
         do {
             node = get_node(doc, nsnode, FALSE);
             if(node)
-                call_event_handlers(doc, event_obj, *get_node_event_target(node), eid, (IDispatch*)HTMLDOMNODE(node));
+                call_event_handlers(doc, event_obj, *get_node_event_target(node), node->cp_container, eid,
+                        (IDispatch*)HTMLDOMNODE(node));
 
             if(!(event_info[eid].flags & EVENT_BUBBLE))
                 break;
@@ -831,14 +923,15 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, nsIDOMNode *target, nsIDOM
             if(NS_SUCCEEDED(nsres) && nsbody) {
                 node = get_node(doc, (nsIDOMNode*)nsbody, FALSE);
                 if(node)
-                    call_event_handlers(doc, event_obj, *get_node_event_target(node), eid, (IDispatch*)HTMLDOMNODE(node));
+                    call_event_handlers(doc, event_obj, *get_node_event_target(node), node->cp_container,
+                            eid, (IDispatch*)HTMLDOMNODE(node));
                 nsIDOMHTMLElement_Release(nsbody);
             }else {
                 ERR("Could not get body: %08x\n", nsres);
             }
         }
 
-        call_event_handlers(doc, event_obj, doc->basedoc.doc_node->node.event_target, eid,
+        call_event_handlers(doc, event_obj, doc->basedoc.doc_node->node.event_target, &doc->basedoc.cp_container, eid,
                 (IDispatch*)HTMLDOC(&doc->basedoc));
         break;
 
@@ -921,7 +1014,8 @@ static BOOL alloc_handler_vector(event_target_t *event_target, eventid_t eid, in
     return TRUE;
 }
 
-static HRESULT set_event_handler_disp(event_target_t **event_target_ptr, HTMLWindow *window, eventid_t eid, IDispatch *disp)
+static HRESULT set_event_handler_disp(event_target_t **event_target_ptr, HTMLDocumentNode *doc,
+        eventid_t eid, IDispatch *disp)
 {
     event_target_t *event_target;
 
@@ -940,23 +1034,23 @@ static HRESULT set_event_handler_disp(event_target_t **event_target_ptr, HTMLWin
         return S_OK;
     IDispatch_AddRef(disp);
 
-    if(window->nswindow && (event_info[eid].flags & EVENT_DEFAULTLISTENER)) {
-        if(!window->event_vector) {
-            window->event_vector = heap_alloc_zero(EVENTID_LAST*sizeof(BOOL));
-            if(!window->event_vector)
+    if(doc->nsdoc && (event_info[eid].flags & EVENT_DEFAULTLISTENER)) {
+        if(!doc->event_vector) {
+            doc->event_vector = heap_alloc_zero(EVENTID_LAST*sizeof(BOOL));
+            if(!doc->event_vector)
                 return E_OUTOFMEMORY;
         }
 
-        if(!window->event_vector[eid]) {
-            window->event_vector[eid] = TRUE;
-            add_nsevent_listener(window, event_info[eid].name);
+        if(!doc->event_vector[eid]) {
+            doc->event_vector[eid] = TRUE;
+            add_nsevent_listener(doc, event_info[eid].name);
         }
     }
 
     return S_OK;
 }
 
-HRESULT set_event_handler(event_target_t **event_target, HTMLDocument *doc, eventid_t eid, VARIANT *var)
+HRESULT set_event_handler(event_target_t **event_target, HTMLDocumentNode *doc, eventid_t eid, VARIANT *var)
 {
     switch(V_VT(var)) {
     case VT_NULL:
@@ -967,7 +1061,7 @@ HRESULT set_event_handler(event_target_t **event_target, HTMLDocument *doc, even
         break;
 
     case VT_DISPATCH:
-        return set_event_handler_disp(event_target, doc->window, eid, V_DISPATCH(var));
+        return set_event_handler_disp(event_target, doc, eid, V_DISPATCH(var));
 
     default:
         FIXME("not supported vt=%d\n", V_VT(var));
@@ -1048,7 +1142,7 @@ void check_event_attr(HTMLDocumentNode *doc, nsIDOMElement *nselem)
             disp = script_parse_event(doc->basedoc.window, attr_value);
             if(disp) {
                 node = get_node(doc, (nsIDOMNode*)nselem, TRUE);
-                set_event_handler_disp(get_node_event_target(node), node->doc->basedoc.window, i, disp);
+                set_event_handler_disp(get_node_event_target(node), node->doc, i, disp);
                 IDispatch_Release(disp);
             }
         }
