@@ -24,6 +24,7 @@
 #include "winbase.h"
 #include "winuser.h"
 #include "ole2.h"
+#include "mshtmdid.h"
 
 #include "wine/debug.h"
 #include "wine/unicode.h"
@@ -39,6 +40,7 @@ static struct list window_list = LIST_INIT(window_list);
 static void window_set_docnode(HTMLWindow *window, HTMLDocumentNode *doc_node)
 {
     if(window->doc) {
+        abort_document_bindings(window->doc);
         window->doc->basedoc.window = NULL;
         htmldoc_release(&window->doc->basedoc);
     }
@@ -99,6 +101,22 @@ static void release_children(HTMLWindow *This)
     }
 }
 
+static HRESULT get_location(HTMLWindow *This, HTMLLocation **ret)
+{
+    if(This->location) {
+        IHTMLLocation_AddRef(HTMLLOCATION(This->location));
+    }else {
+        HRESULT hres;
+
+        hres = HTMLLocation_Create(This, &This->location);
+        if(FAILED(hres))
+            return hres;
+    }
+
+    *ret = This->location;
+    return S_OK;
+}
+
 #define HTMLWINDOW2_THIS(iface) DEFINE_THIS(HTMLWindow, HTMLWindow2, iface)
 
 static HRESULT WINAPI HTMLWindow2_QueryInterface(IHTMLWindow2 *iface, REFIID riid, void **ppv)
@@ -123,8 +141,11 @@ static HRESULT WINAPI HTMLWindow2_QueryInterface(IHTMLWindow2 *iface, REFIID rii
         TRACE("(%p)->(IID_IHTMLWindow2 %p)\n", This, ppv);
         *ppv = HTMLWINDOW2(This);
     }else if(IsEqualGUID(&IID_IHTMLWindow3, riid)) {
-        TRACE("(%p)->(IID_IHTMLWindow2 %p)\n", This, ppv);
+        TRACE("(%p)->(IID_IHTMLWindow3 %p)\n", This, ppv);
         *ppv = HTMLWINDOW3(This);
+    }else if(IsEqualGUID(&IID_IHTMLWindow4, riid)) {
+        TRACE("(%p)->(IID_IHTMLWindow4 %p)\n", This, ppv);
+        *ppv = HTMLWINDOW4(This);
     }else if(dispex_query_interface(&This->dispex, riid, ppv)) {
         return *ppv ? S_OK : E_NOINTERFACE;
     }
@@ -158,9 +179,15 @@ static ULONG WINAPI HTMLWindow2_Release(IHTMLWindow2 *iface)
     if(!ref) {
         DWORD i;
 
+        remove_target_tasks(This->task_magic);
+        set_window_bscallback(This, NULL);
+        set_current_mon(This, NULL);
         window_set_docnode(This, NULL);
         release_children(This);
 
+        if(This->frame_element)
+            This->frame_element->content_window = NULL;
+
         if(This->option_factory) {
             This->option_factory->window = NULL;
             IHTMLOptionElementFactory_Release(HTMLOPTFACTORY(This->option_factory));
@@ -176,13 +203,23 @@ static ULONG WINAPI HTMLWindow2_Release(IHTMLWindow2 *iface)
             IHTMLLocation_Release(HTMLLOCATION(This->location));
         }
 
+        if(This->screen)
+            IHTMLScreen_Release(This->screen);
+
         if(This->event_target)
             release_event_target(This->event_target);
         for(i=0; i < This->global_prop_cnt; i++)
             heap_free(This->global_props[i].name);
+
+        This->window_ref->window = NULL;
+        windowref_release(This->window_ref);
+
         heap_free(This->global_props);
-        heap_free(This->event_vector);
         release_script_hosts(This);
+
+        if(This->nswindow)
+            nsIDOMWindow_Release(This->nswindow);
+
         list_remove(&This->entry);
         release_dispex(&This->dispex);
         heap_free(This);
@@ -225,11 +262,122 @@ static HRESULT WINAPI HTMLWindow2_Invoke(IHTMLWindow2 *iface, DISPID dispIdMembe
             pVarResult, pExcepInfo, puArgErr);
 }
 
+static HRESULT get_frame_by_index(nsIDOMWindowCollection *nsFrames, PRUint32 index, HTMLWindow **ret)
+{
+    PRUint32 length;
+    nsIDOMWindow *nsWindow;
+    nsresult nsres;
+
+    nsres = nsIDOMWindowCollection_GetLength(nsFrames, &length);
+    if(NS_FAILED(nsres)) {
+        FIXME("nsIDOMWindowCollection_GetLength failed: 0x%08x\n", nsres);
+        return E_FAIL;
+    }
+
+    if(index >= length)
+        return DISP_E_MEMBERNOTFOUND;
+
+    nsres = nsIDOMWindowCollection_Item(nsFrames, index, &nsWindow);
+    if(NS_FAILED(nsres)) {
+        FIXME("nsIDOMWindowCollection_Item failed: 0x%08x\n", nsres);
+        return E_FAIL;
+    }
+
+    *ret = nswindow_to_window(nsWindow);
+
+    nsIDOMWindow_Release(nsWindow);
+
+    return S_OK;
+}
+
 static HRESULT WINAPI HTMLWindow2_item(IHTMLWindow2 *iface, VARIANT *pvarIndex, VARIANT *pvarResult)
 {
     HTMLWindow *This = HTMLWINDOW2_THIS(iface);
-    FIXME("(%p)->(%p %p)\n", This, pvarIndex, pvarResult);
-    return E_NOTIMPL;
+    nsIDOMWindowCollection *nsFrames;
+    HTMLWindow *window;
+    HRESULT hres;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p %p)\n", This, pvarIndex, pvarResult);
+
+    nsres = nsIDOMWindow_GetFrames(This->nswindow, &nsFrames);
+    if(NS_FAILED(nsres)) {
+        FIXME("nsIDOMWindow_GetFrames failed: 0x%08x\n", nsres);
+        return E_FAIL;
+    }
+
+    if(V_VT(pvarIndex) == VT_I4) {
+        int index = V_I4(pvarIndex);
+        TRACE("Getting index %d\n", index);
+        if(index < 0) {
+            hres = DISP_E_MEMBERNOTFOUND;
+            goto cleanup;
+        }
+        hres = get_frame_by_index(nsFrames, index, &window);
+        if(FAILED(hres))
+            goto cleanup;
+    }else if(V_VT(pvarIndex) == VT_UINT) {
+        unsigned int index = V_UINT(pvarIndex);
+        TRACE("Getting index %u\n", index);
+        hres = get_frame_by_index(nsFrames, index, &window);
+        if(FAILED(hres))
+            goto cleanup;
+    }else if(V_VT(pvarIndex) == VT_BSTR) {
+        BSTR str = V_BSTR(pvarIndex);
+        PRUint32 length, i;
+
+        TRACE("Getting name %s\n", wine_dbgstr_w(str));
+
+        nsres = nsIDOMWindowCollection_GetLength(nsFrames, &length);
+
+        window = NULL;
+        for(i = 0; i < length && !window; ++i) {
+            HTMLWindow *cur_window;
+            nsIDOMWindow *nsWindow;
+            BSTR id;
+
+            nsres = nsIDOMWindowCollection_Item(nsFrames, i, &nsWindow);
+            if(NS_FAILED(nsres)) {
+                FIXME("nsIDOMWindowCollection_Item failed: 0x%08x\n", nsres);
+                hres = E_FAIL;
+                goto cleanup;
+            }
+
+            cur_window = nswindow_to_window(nsWindow);
+
+            nsIDOMWindow_Release(nsWindow);
+
+            hres = IHTMLElement_get_id(HTMLELEM(&cur_window->frame_element->element), &id);
+            if(FAILED(hres)) {
+                FIXME("IHTMLElement_get_id failed: 0x%08x\n", hres);
+                goto cleanup;
+            }
+
+            if(!strcmpW(id, str))
+                window = cur_window;
+
+            SysFreeString(id);
+        }
+
+        if(!window) {
+            hres = DISP_E_MEMBERNOTFOUND;
+            goto cleanup;
+        }
+    }else {
+        hres = E_INVALIDARG;
+        goto cleanup;
+    }
+
+    IHTMLWindow2_AddRef(HTMLWINDOW2(window));
+    V_VT(pvarResult) = VT_DISPATCH;
+    V_DISPATCH(pvarResult) = (IDispatch*)window;
+
+    hres = S_OK;
+
+cleanup:
+    nsIDOMWindowCollection_Release(nsFrames);
+
+    return hres;
 }
 
 static HRESULT WINAPI HTMLWindow2_get_length(IHTMLWindow2 *iface, LONG *p)
@@ -261,8 +409,12 @@ static HRESULT WINAPI HTMLWindow2_get_length(IHTMLWindow2 *iface, LONG *p)
 static HRESULT WINAPI HTMLWindow2_get_frames(IHTMLWindow2 *iface, IHTMLFramesCollection2 **p)
 {
     HTMLWindow *This = HTMLWINDOW2_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    FIXME("(%p)->(%p): semi-stub\n", This, p);
+
+    /* FIXME: Should return a separate Window object */
+    *p = (IHTMLFramesCollection2*)HTMLWINDOW2(This);
+    HTMLWindow2_AddRef(iface);
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLWindow2_put_defaultStatus(IHTMLWindow2 *iface, BSTR v)
@@ -465,20 +617,16 @@ static HRESULT WINAPI HTMLWindow2_get_Image(IHTMLWindow2 *iface, IHTMLImageEleme
 static HRESULT WINAPI HTMLWindow2_get_location(IHTMLWindow2 *iface, IHTMLLocation **p)
 {
     HTMLWindow *This = HTMLWINDOW2_THIS(iface);
+    HTMLLocation *location;
+    HRESULT hres;
 
     TRACE("(%p)->(%p)\n", This, p);
 
-    if(This->location) {
-        IHTMLLocation_AddRef(HTMLLOCATION(This->location));
-    }else {
-        HRESULT hres;
+    hres = get_location(This, &location);
+    if(FAILED(hres))
+        return hres;
 
-        hres = HTMLLocation_Create(This, &This->location);
-        if(FAILED(hres))
-            return hres;
-    }
-
-    *p = HTMLLOCATION(This->location);
+    *p = HTMLLOCATION(location);
     return S_OK;
 }
 
@@ -571,8 +719,15 @@ static HRESULT WINAPI HTMLWindow2_get_name(IHTMLWindow2 *iface, BSTR *p)
 static HRESULT WINAPI HTMLWindow2_get_parent(IHTMLWindow2 *iface, IHTMLWindow2 **p)
 {
     HTMLWindow *This = HTMLWINDOW2_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    TRACE("(%p)->(%p)\n", This, p);
+
+    if(This->parent) {
+        *p = HTMLWINDOW2(This->parent);
+        IHTMLWindow2_AddRef(*p);
+    }else
+        *p = NULL;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLWindow2_open(IHTMLWindow2 *iface, BSTR url, BSTR name,
@@ -598,9 +753,16 @@ static HRESULT WINAPI HTMLWindow2_get_self(IHTMLWindow2 *iface, IHTMLWindow2 **p
 
 static HRESULT WINAPI HTMLWindow2_get_top(IHTMLWindow2 *iface, IHTMLWindow2 **p)
 {
-    HTMLWindow *This = HTMLWINDOW2_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    HTMLWindow *This = HTMLWINDOW2_THIS(iface), *curr;
+    TRACE("(%p)->(%p)\n", This, p);
+
+    curr = This;
+    while(curr->parent)
+        curr = curr->parent;
+    *p = HTMLWINDOW2(curr);
+    IHTMLWindow2_AddRef(*p);
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLWindow2_get_window(IHTMLWindow2 *iface, IHTMLWindow2 **p)
@@ -811,8 +973,20 @@ static HRESULT WINAPI HTMLWindow2_showHelp(IHTMLWindow2 *iface, BSTR helpURL, VA
 static HRESULT WINAPI HTMLWindow2_get_screen(IHTMLWindow2 *iface, IHTMLScreen **p)
 {
     HTMLWindow *This = HTMLWINDOW2_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    if(!This->screen) {
+        HRESULT hres;
+
+        hres = HTMLScreen_Create(&This->screen);
+        if(FAILED(hres))
+            return hres;
+    }
+
+    *p = This->screen;
+    IHTMLScreen_AddRef(This->screen);
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLWindow2_get_Option(IHTMLWindow2 *iface, IHTMLOptionElementFactory **p)
@@ -998,8 +1172,7 @@ static HRESULT HTMLWindow_invoke(IUnknown *iface, DISPID id, LCID lcid, WORD fla
         VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
 {
     HTMLWindow *This = HTMLWINDOW2_THIS(iface);
-    IDispatchEx *dispex;
-    IDispatch *disp;
+    global_prop_t *prop;
     DWORD idx;
     HRESULT hres;
 
@@ -1007,24 +1180,51 @@ static HRESULT HTMLWindow_invoke(IUnknown *iface, DISPID id, LCID lcid, WORD fla
     if(idx >= This->global_prop_cnt)
         return DISP_E_MEMBERNOTFOUND;
 
-    disp = get_script_disp(This->global_props[idx].script_host);
-    if(!disp)
-        return E_UNEXPECTED;
+    prop = This->global_props+idx;
 
-    hres = IDispatch_QueryInterface(disp, &IID_IDispatchEx, (void**)&dispex);
-    if(SUCCEEDED(hres)) {
-        TRACE("%s >>>\n", debugstr_w(This->global_props[idx].name));
-        hres = IDispatchEx_InvokeEx(dispex, This->global_props[idx].id, lcid, flags, params, res, ei, caller);
-        if(hres == S_OK)
-            TRACE("%s <<<\n", debugstr_w(This->global_props[idx].name));
-        else
-            WARN("%s <<< %08x\n", debugstr_w(This->global_props[idx].name), hres);
-        IDispatchEx_Release(dispex);
-    }else {
-        FIXME("No IDispatchEx\n");
+    switch(prop->type) {
+    case GLOBAL_SCRIPTVAR: {
+        IDispatchEx *dispex;
+        IDispatch *disp;
+
+        disp = get_script_disp(prop->script_host);
+        if(!disp)
+            return E_UNEXPECTED;
+
+        hres = IDispatch_QueryInterface(disp, &IID_IDispatchEx, (void**)&dispex);
+        if(SUCCEEDED(hres)) {
+            TRACE("%s >>>\n", debugstr_w(prop->name));
+            hres = IDispatchEx_InvokeEx(dispex, prop->id, lcid, flags, params, res, ei, caller);
+            if(hres == S_OK)
+                TRACE("%s <<<\n", debugstr_w(prop->name));
+            else
+                WARN("%s <<< %08x\n", debugstr_w(prop->name), hres);
+            IDispatchEx_Release(dispex);
+        }else {
+            FIXME("No IDispatchEx\n");
+        }
+        IDispatch_Release(disp);
+        break;
+    }
+    case GLOBAL_ELEMENTVAR: {
+        IHTMLElement *elem;
+
+        hres = IHTMLDocument3_getElementById(HTMLDOC3(&This->doc->basedoc), prop->name, &elem);
+        if(FAILED(hres))
+            return hres;
+
+        if(!elem)
+            return DISP_E_MEMBERNOTFOUND;
+
+        V_VT(res) = VT_DISPATCH;
+        V_DISPATCH(res) = (IDispatch*)elem;
+        break;
+    }
+    default:
+        ERR("invalid type %d\n", prop->type);
+        hres = DISP_E_MEMBERNOTFOUND;
     }
 
-    IDispatch_Release(disp);
     return hres;
 }
 
@@ -1321,6 +1521,99 @@ static const IHTMLWindow3Vtbl HTMLWindow3Vtbl = {
     HTMLWindow3_showModelessDialog
 };
 
+#define HTMLWINDOW4_THIS(iface) DEFINE_THIS(HTMLWindow, HTMLWindow4, iface)
+
+static HRESULT WINAPI HTMLWindow4_QueryInterface(IHTMLWindow4 *iface, REFIID riid, void **ppv)
+{
+    HTMLWindow *This = HTMLWINDOW4_THIS(iface);
+
+    return IHTMLWindow2_QueryInterface(HTMLWINDOW2(This), riid, ppv);
+}
+
+static ULONG WINAPI HTMLWindow4_AddRef(IHTMLWindow4 *iface)
+{
+    HTMLWindow *This = HTMLWINDOW4_THIS(iface);
+
+    return IHTMLWindow2_AddRef(HTMLWINDOW2(This));
+}
+
+static ULONG WINAPI HTMLWindow4_Release(IHTMLWindow4 *iface)
+{
+    HTMLWindow *This = HTMLWINDOW4_THIS(iface);
+
+    return IHTMLWindow2_Release(HTMLWINDOW2(This));
+}
+
+static HRESULT WINAPI HTMLWindow4_GetTypeInfoCount(IHTMLWindow4 *iface, UINT *pctinfo)
+{
+    HTMLWindow *This = HTMLWINDOW4_THIS(iface);
+
+    return IDispatchEx_GetTypeInfoCount(DISPATCHEX(This), pctinfo);
+}
+
+static HRESULT WINAPI HTMLWindow4_GetTypeInfo(IHTMLWindow4 *iface, UINT iTInfo,
+                                              LCID lcid, ITypeInfo **ppTInfo)
+{
+    HTMLWindow *This = HTMLWINDOW4_THIS(iface);
+
+    return IDispatchEx_GetTypeInfo(DISPATCHEX(This), iTInfo, lcid, ppTInfo);
+}
+
+static HRESULT WINAPI HTMLWindow4_GetIDsOfNames(IHTMLWindow4 *iface, REFIID riid,
+                                                LPOLESTR *rgszNames, UINT cNames,
+                                                LCID lcid, DISPID *rgDispId)
+{
+    HTMLWindow *This = HTMLWINDOW4_THIS(iface);
+
+    return IDispatchEx_GetIDsOfNames(DISPATCHEX(This), riid, rgszNames, cNames, lcid, rgDispId);
+}
+
+static HRESULT WINAPI HTMLWindow4_Invoke(IHTMLWindow4 *iface, DISPID dispIdMember,
+                            REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
+                            VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
+{
+    HTMLWindow *This = HTMLWINDOW4_THIS(iface);
+
+    return IDispatchEx_Invoke(DISPATCHEX(This), dispIdMember, riid, lcid, wFlags, pDispParams,
+            pVarResult, pExcepInfo, puArgErr);
+}
+
+static HRESULT WINAPI HTMLWindow4_createPopup(IHTMLWindow4 *iface, VARIANT *varArgIn,
+                            IDispatch **ppPopup)
+{
+    HTMLWindow *This = HTMLWINDOW4_THIS(iface);
+    FIXME("(%p)->(%p %p)\n", This, varArgIn, ppPopup);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI HTMLWindow4_get_frameElement(IHTMLWindow4 *iface, IHTMLFrameBase **p)
+{
+    HTMLWindow *This = HTMLWINDOW4_THIS(iface);
+    TRACE("(%p)->(%p)\n", This, p);
+
+    if(This->frame_element) {
+        *p = HTMLFRAMEBASE(This->frame_element);
+        IHTMLFrameBase_AddRef(*p);
+    }else
+        *p = NULL;
+
+    return S_OK;
+}
+
+#undef HTMLWINDOW4_THIS
+
+static const IHTMLWindow4Vtbl HTMLWindow4Vtbl = {
+    HTMLWindow4_QueryInterface,
+    HTMLWindow4_AddRef,
+    HTMLWindow4_Release,
+    HTMLWindow4_GetTypeInfoCount,
+    HTMLWindow4_GetTypeInfo,
+    HTMLWindow4_GetIDsOfNames,
+    HTMLWindow4_Invoke,
+    HTMLWindow4_createPopup,
+    HTMLWindow4_get_frameElement
+};
+
 #define DISPEX_THIS(iface) DEFINE_THIS(HTMLWindow, IDispatchEx, iface)
 
 static HRESULT WINAPI WindowDispEx_QueryInterface(IDispatchEx *iface, REFIID riid, void **ppv)
@@ -1392,12 +1685,45 @@ static HRESULT WINAPI WindowDispEx_Invoke(IDispatchEx *iface, DISPID dispIdMembe
                               pVarResult, pExcepInfo, puArgErr);
 }
 
+static global_prop_t *alloc_global_prop(HTMLWindow *This, global_prop_type_t type, BSTR name)
+{
+    if(This->global_prop_cnt == This->global_prop_size) {
+        global_prop_t *new_props;
+        DWORD new_size;
+
+        if(This->global_props) {
+            new_size = This->global_prop_size*2;
+            new_props = heap_realloc(This->global_props, new_size*sizeof(global_prop_t));
+        }else {
+            new_size = 16;
+            new_props = heap_alloc(new_size*sizeof(global_prop_t));
+        }
+        if(!new_props)
+            return NULL;
+        This->global_props = new_props;
+        This->global_prop_size = new_size;
+    }
+
+    This->global_props[This->global_prop_cnt].name = heap_strdupW(name);
+    if(!This->global_props[This->global_prop_cnt].name)
+        return NULL;
+
+    This->global_props[This->global_prop_cnt].type = type;
+    return This->global_props + This->global_prop_cnt++;
+}
+
+static inline DWORD prop_to_dispid(HTMLWindow *This, global_prop_t *prop)
+{
+    return MSHTML_DISPID_CUSTOM_MIN + (prop-This->global_props);
+}
+
 static HRESULT WINAPI WindowDispEx_GetDispID(IDispatchEx *iface, BSTR bstrName, DWORD grfdex, DISPID *pid)
 {
     HTMLWindow *This = DISPEX_THIS(iface);
     ScriptHost *script_host;
     DISPID id;
     DWORD i;
+    HRESULT hres;
 
     TRACE("(%p)->(%s %x %p)\n", This, debugstr_w(bstrName), grfdex, pid);
 
@@ -1410,35 +1736,41 @@ static HRESULT WINAPI WindowDispEx_GetDispID(IDispatchEx *iface, BSTR bstrName,
     }
 
     if(find_global_prop(This, bstrName, grfdex, &script_host, &id)) {
-        if(This->global_prop_cnt == This->global_prop_size) {
-            global_prop_t *new_props;
-            DWORD new_size;
+        global_prop_t *prop;
 
-            if(This->global_props) {
-                new_size = This->global_prop_size*2;
-                new_props = heap_realloc(This->global_props, new_size*sizeof(global_prop_t));
-            }else {
-                new_size = 16;
-                new_props = heap_alloc(new_size*sizeof(global_prop_t));
-            }
-            if(!new_props)
-                return E_OUTOFMEMORY;
-            This->global_props = new_props;
-            This->global_prop_size = new_size;
-        }
-
-        This->global_props[This->global_prop_cnt].name = heap_strdupW(bstrName);
-        if(!This->global_props[This->global_prop_cnt].name)
+        prop = alloc_global_prop(This, GLOBAL_SCRIPTVAR, bstrName);
+        if(!prop)
             return E_OUTOFMEMORY;
 
-        This->global_props[This->global_prop_cnt].script_host = script_host;
-        This->global_props[This->global_prop_cnt].id = id;
+        prop->script_host = script_host;
+        prop->id = id;
 
-        *pid = MSHTML_DISPID_CUSTOM_MIN + (This->global_prop_cnt++);
+        *pid = prop_to_dispid(This, prop);
         return S_OK;
     }
 
-    return IDispatchEx_GetDispID(DISPATCHEX(&This->dispex), bstrName, grfdex, pid);
+    hres = IDispatchEx_GetDispID(DISPATCHEX(&This->dispex), bstrName, grfdex, pid);
+    if(hres != DISP_E_UNKNOWNNAME)
+        return hres;
+
+    if(This->doc) {
+        global_prop_t *prop;
+        IHTMLElement *elem;
+
+        hres = IHTMLDocument3_getElementById(HTMLDOC3(&This->doc->basedoc), bstrName, &elem);
+        if(SUCCEEDED(hres) && elem) {
+            IHTMLElement_Release(elem);
+
+            prop = alloc_global_prop(This, GLOBAL_ELEMENTVAR, bstrName);
+            if(!prop)
+                return E_OUTOFMEMORY;
+
+            *pid = prop_to_dispid(This, prop);
+            return S_OK;
+        }
+    }
+
+    return DISP_E_UNKNOWNNAME;
 }
 
 static HRESULT WINAPI WindowDispEx_InvokeEx(IDispatchEx *iface, DISPID id, LCID lcid, WORD wFlags, DISPPARAMS *pdp,
@@ -1448,6 +1780,21 @@ static HRESULT WINAPI WindowDispEx_InvokeEx(IDispatchEx *iface, DISPID id, LCID
 
     TRACE("(%p)->(%x %x %x %p %p %p %p)\n", This, id, lcid, wFlags, pdp, pvarRes, pei, pspCaller);
 
+    if(id == DISPID_IHTMLWINDOW2_LOCATION && (wFlags & DISPATCH_PROPERTYPUT)) {
+        HTMLLocation *location;
+        HRESULT hres;
+
+        TRACE("forwarding to location.href\n");
+
+        hres = get_location(This, &location);
+        if(FAILED(hres))
+            return hres;
+
+        hres = IDispatchEx_InvokeEx(DISPATCHEX(&location->dispex), DISPID_VALUE, lcid, wFlags, pdp, pvarRes, pei, pspCaller);
+        IHTMLLocation_Release(HTMLLOCATION(location));
+        return hres;
+    }
+
     return IDispatchEx_InvokeEx(DISPATCHEX(&This->dispex), id, lcid, wFlags, pdp, pvarRes, pei, pspCaller);
 }
 
@@ -1529,6 +1876,7 @@ static const IDispatchExVtbl WindowDispExVtbl = {
 static const tid_t HTMLWindow_iface_tids[] = {
     IHTMLWindow2_tid,
     IHTMLWindow3_tid,
+    IHTMLWindow4_tid,
     0
 };
 
@@ -1553,12 +1901,22 @@ HRESULT HTMLWindow_Create(HTMLDocumentObj *doc_obj, nsIDOMWindow *nswindow, HTML
     if(!window)
         return E_OUTOFMEMORY;
 
+    window->window_ref = heap_alloc(sizeof(windowref_t));
+    if(!window->window_ref) {
+        heap_free(window);
+        return E_OUTOFMEMORY;
+    }
+
     window->lpHTMLWindow2Vtbl = &HTMLWindow2Vtbl;
     window->lpHTMLWindow3Vtbl = &HTMLWindow3Vtbl;
+    window->lpHTMLWindow4Vtbl = &HTMLWindow4Vtbl;
     window->lpIDispatchExVtbl = &WindowDispExVtbl;
     window->ref = 1;
     window->doc_obj = doc_obj;
 
+    window->window_ref->window = window;
+    window->window_ref->ref = 1;
+
     init_dispex(&window->dispex, (IUnknown*)HTMLWINDOW2(window), &HTMLWindow_dispex);
 
     if(nswindow) {
@@ -1566,9 +1924,11 @@ HRESULT HTMLWindow_Create(HTMLDocumentObj *doc_obj, nsIDOMWindow *nswindow, HTML
         window->nswindow = nswindow;
     }
 
-    window->scriptmode = SCRIPTMODE_GECKO;
+    window->scriptmode = parent ? parent->scriptmode : SCRIPTMODE_GECKO;
+    window->readystate = READYSTATE_UNINITIALIZED;
     list_init(&window->script_hosts);
 
+    window->task_magic = get_task_target_magic();
     update_window_doc(window);
 
     list_init(&window->children);
