@@ -18,6 +18,7 @@
 
 #include "wingdi.h"
 #include "docobj.h"
+#include "comcat.h"
 #include "mshtml.h"
 #include "mshtmhst.h"
 #include "hlink.h"
@@ -66,6 +67,7 @@ typedef enum {
     DispHTMLDocument_tid,
     DispHTMLDOMTextNode_tid,
     DispHTMLElementCollection_tid,
+    DispHTMLFormElement_tid,
     DispHTMLGenericElement_tid,
     DispHTMLIFrame_tid,
     DispHTMLImg_tid,
@@ -73,12 +75,14 @@ typedef enum {
     DispHTMLLocation_tid,
     DispHTMLNavigator_tid,
     DispHTMLOptionElement_tid,
+    DispHTMLScreen_tid,
     DispHTMLSelectElement_tid,
     DispHTMLStyle_tid,
     DispHTMLTable_tid,
     DispHTMLTableRow_tid,
     DispHTMLUnknownElement_tid,
     DispHTMLWindow2_tid,
+    HTMLDocumentEvents_tid,
     IHTMLAnchorElement_tid,
     IHTMLBodyElement_tid,
     IHTMLBodyElement2_tid,
@@ -101,6 +105,7 @@ typedef enum {
     IHTMLElement4_tid,
     IHTMLElementCollection_tid,
     IHTMLEventObj_tid,
+    IHTMLFormElement_tid,
     IHTMLFrameBase_tid,
     IHTMLFrameBase2_tid,
     IHTMLGenericElement_tid,
@@ -109,6 +114,7 @@ typedef enum {
     IHTMLInputElement_tid,
     IHTMLLocation_tid,
     IHTMLOptionElement_tid,
+    IHTMLScreen_tid,
     IHTMLSelectElement_tid,
     IHTMLStyle_tid,
     IHTMLStyle2_tid,
@@ -120,6 +126,7 @@ typedef enum {
     IHTMLUniqueName_tid,
     IHTMLWindow2_tid,
     IHTMLWindow3_tid,
+    IHTMLWindow4_tid,
     IOmNavigator_tid,
     LAST_tid
 } tid_t;
@@ -156,10 +163,11 @@ void init_dispex(DispatchEx*,IUnknown*,dispex_static_data_t*);
 void release_dispex(DispatchEx*);
 BOOL dispex_query_interface(DispatchEx*,REFIID,void**);
 HRESULT dispex_get_dprop_ref(DispatchEx*,const WCHAR*,BOOL,VARIANT**);
+HRESULT get_dispids(tid_t,DWORD*,DISPID**);
 
-typedef struct HTMLWindow HTMLWindow;
 typedef struct HTMLDocumentNode HTMLDocumentNode;
 typedef struct HTMLDocumentObj HTMLDocumentObj;
+typedef struct HTMLFrameBase HTMLFrameBase;
 
 typedef enum {
     SCRIPTMODE_GECKO,
@@ -168,7 +176,13 @@ typedef enum {
 
 typedef struct ScriptHost ScriptHost;
 
+typedef enum {
+    GLOBAL_SCRIPTVAR,
+    GLOBAL_ELEMENTVAR
+} global_prop_type_t;
+
 typedef struct {
+    global_prop_type_t type;
     WCHAR *name;
     ScriptHost *script_host;
     DISPID id;
@@ -200,22 +214,36 @@ struct HTMLLocation {
     HTMLWindow *window;
 };
 
+typedef struct {
+    HTMLWindow *window;
+    LONG ref;
+}  windowref_t;
+
 struct HTMLWindow {
     DispatchEx dispex;
     const IHTMLWindow2Vtbl *lpHTMLWindow2Vtbl;
     const IHTMLWindow3Vtbl *lpHTMLWindow3Vtbl;
+    const IHTMLWindow4Vtbl *lpHTMLWindow4Vtbl;
     const IDispatchExVtbl  *lpIDispatchExVtbl;
 
     LONG ref;
 
+    windowref_t *window_ref;
+    LONG task_magic;
+
     HTMLDocumentNode *doc;
     HTMLDocumentObj *doc_obj;
     nsIDOMWindow *nswindow;
     HTMLWindow *parent;
+    HTMLFrameBase *frame_element;
+    READYSTATE readystate;
+
+    nsChannelBSC *bscallback;
+    IMoniker *mon;
+    LPOLESTR url;
 
     event_target_t *event_target;
     IHTMLEventObj *event;
-    BOOL *event_vector;
 
     SCRIPTMODE scriptmode;
     struct list script_hosts;
@@ -223,6 +251,7 @@ struct HTMLWindow {
     HTMLOptionElementFactory *option_factory;
     HTMLImageElementFactory *image_factory;
     HTMLLocation *location;
+    IHTMLScreen *screen;
 
     global_prop_t *global_props;
     DWORD global_prop_cnt;
@@ -240,10 +269,17 @@ typedef enum {
 } USERMODE;
 
 typedef struct {
+    tid_t tid;
+    DWORD id_cnt;
+    DISPID *ids;
+} cp_static_data_t;
+
+typedef struct ConnectionPointContainer {
     const IConnectionPointContainerVtbl  *lpConnectionPointContainerVtbl;
 
     ConnectionPoint *cp_list;
     IUnknown *outer;
+    struct ConnectionPointContainer *forward_container;
 } ConnectionPointContainer;
 
 struct ConnectionPoint {
@@ -259,6 +295,7 @@ struct ConnectionPoint {
     DWORD sinks_size;
 
     const IID *iid;
+    cp_static_data_t *data;
 
     ConnectionPoint *next;
 };
@@ -295,6 +332,8 @@ struct HTMLDocument {
 
     HTMLWindow *window;
 
+    LONG task_magic;
+
     ConnectionPointContainer cp_container;
     ConnectionPoint cp_htmldocevents;
     ConnectionPoint cp_htmldocevents2;
@@ -346,23 +385,11 @@ struct HTMLDocumentObj {
     BOOL focus;
 
     USERMODE usermode;
-    READYSTATE readystate;
     LPWSTR mime;
 
     DWORD update;
-
-    /* FIXME: probably should be in document node object */
-    nsChannelBSC *bscallback;
-    IMoniker *mon;
-    LPOLESTR url;
-    struct list bindings;
 };
 
-typedef struct {
-    const nsIDOMEventListenerVtbl      *lpDOMEventListenerVtbl;
-    NSContainer *This;
-} nsEventListener;
-
 struct NSContainer {
     const nsIWebBrowserChromeVtbl       *lpWebBrowserChromeVtbl;
     const nsIContextMenuListenerVtbl    *lpContextMenuListenerVtbl;
@@ -373,12 +400,6 @@ struct NSContainer {
     const nsIWeakReferenceVtbl          *lpWeakReferenceVtbl;
     const nsISupportsWeakReferenceVtbl  *lpSupportsWeakReferenceVtbl;
 
-    nsEventListener blur_listener;
-    nsEventListener focus_listener;
-    nsEventListener keypress_listener;
-    nsEventListener load_listener;
-    nsEventListener htmlevent_listener;
-
     nsIWebBrowser *webbrowser;
     nsIWebNavigation *navigation;
     nsIBaseWindow *window;
@@ -420,6 +441,7 @@ typedef struct {
     char *content_type;
     char *charset;
     PRUint32 response_status;
+    UINT url_scheme;
 } nsChannel;
 
 typedef struct {
@@ -429,6 +451,11 @@ typedef struct {
     HRESULT (*call_event)(HTMLDOMNode*,DWORD,BOOL*);
     HRESULT (*put_disabled)(HTMLDOMNode*,VARIANT_BOOL);
     HRESULT (*get_disabled)(HTMLDOMNode*,VARIANT_BOOL*);
+    HRESULT (*get_document)(HTMLDOMNode*,IDispatch**);
+    HRESULT (*get_readystate)(HTMLDOMNode*,BSTR*);
+    HRESULT (*get_dispid)(HTMLDOMNode*,BSTR,DWORD,DISPID*);
+    HRESULT (*invoke)(HTMLDOMNode*,DISPID,LCID,WORD,DISPPARAMS*,VARIANT*,EXCEPINFO*,IServiceProvider*);
+    HRESULT (*bind_to_tree)(HTMLDOMNode*);
 } NodeImplVtbl;
 
 struct HTMLDOMNode {
@@ -442,6 +469,7 @@ struct HTMLDOMNode {
     nsIDOMNode *nsnode;
     HTMLDocumentNode *doc;
     event_target_t *event_target;
+    ConnectionPointContainer *cp_container;
 
     HTMLDOMNode *next;
 };
@@ -465,6 +493,15 @@ typedef struct {
     ConnectionPoint cp;
 } HTMLTextContainer;
 
+struct HTMLFrameBase {
+    HTMLElement element;
+
+    const IHTMLFrameBaseVtbl   *lpIHTMLFrameBaseVtbl;
+    const IHTMLFrameBase2Vtbl  *lpIHTMLFrameBase2Vtbl;
+
+    HTMLWindow *content_window;
+};
+
 typedef struct _mutation_queue_t {
     DWORD type;
     nsISupports *nsiface;
@@ -472,6 +509,8 @@ typedef struct _mutation_queue_t {
     struct _mutation_queue_t *next;
 } mutation_queue_t;
 
+typedef struct nsDocumentEventListener nsDocumentEventListener;
+
 struct HTMLDocumentNode {
     HTMLDOMNode node;
     HTMLDocument basedoc;
@@ -488,16 +527,21 @@ struct HTMLDocumentNode {
     BOOL content_ready;
 
     IInternetSecurityManager *secmgr;
+    ICatInformation *catmgr;
+    nsDocumentEventListener *nsevent_listener;
+    BOOL *event_vector;
 
     mutation_queue_t *mutation_queue;
     mutation_queue_t *mutation_queue_tail;
 
+    struct list bindings;
     struct list selection_list;
     struct list range_list;
 };
 
 #define HTMLWINDOW2(x)   ((IHTMLWindow2*)                 &(x)->lpHTMLWindow2Vtbl)
 #define HTMLWINDOW3(x)   ((IHTMLWindow3*)                 &(x)->lpHTMLWindow3Vtbl)
+#define HTMLWINDOW4(x)   ((IHTMLWindow4*)                 &(x)->lpHTMLWindow4Vtbl)
 
 #define HTMLDOC(x)       ((IHTMLDocument2*)               &(x)->lpHTMLDocument2Vtbl)
 #define HTMLDOC3(x)      ((IHTMLDocument3*)               &(x)->lpHTMLDocument3Vtbl)
@@ -556,10 +600,12 @@ struct HTMLDocumentNode {
 #define HTMLDOMNODE2(x)  ((IHTMLDOMNode2*)                &(x)->lpHTMLDOMNode2Vtbl)
 
 #define HTMLTEXTCONT(x)  ((IHTMLTextContainer*)           &(x)->lpHTMLTextContainerVtbl)
+#define HTMLFRAMEBASE(x) ((IHTMLFrameBase*)               &(x)->lpIHTMLFrameBaseVtbl)
+#define HTMLFRAMEBASE2(x) ((IHTMLFrameBase2*)             &(x)->lpIHTMLFrameBase2Vtbl)
 
 #define HTMLOPTFACTORY(x)  ((IHTMLOptionElementFactory*)  &(x)->lpHTMLOptionElementFactoryVtbl)
-#define HTMLIMGFACTORY(x)  ((IHTMLImageElementFactory*)  &(x)->lpHTMLImageElementFactoryVtbl)
-#define HTMLLOCATION(x)  ((IHTMLLocation*) &(x)->lpHTMLLocationVtbl)
+#define HTMLIMGFACTORY(x)  ((IHTMLImageElementFactory*)   &(x)->lpHTMLImageElementFactoryVtbl)
+#define HTMLLOCATION(x)    ((IHTMLLocation*)              &(x)->lpHTMLLocationVtbl)
 
 #define DISPATCHEX(x)    ((IDispatchEx*) &(x)->lpIDispatchExVtbl)
 
@@ -582,6 +628,7 @@ HTMLOptionElementFactory *HTMLOptionElementFactory_Create(HTMLWindow*);
 HTMLImageElementFactory *HTMLImageElementFactory_Create(HTMLWindow*);
 HRESULT HTMLLocation_Create(HTMLWindow*,HTMLLocation**);
 IOmNavigator *OmNavigator_Create(void);
+HRESULT HTMLScreen_Create(IHTMLScreen**);
 
 void HTMLDocument_HTMLDocument3_Init(HTMLDocument*);
 void HTMLDocument_HTMLDocument5_Init(HTMLDocument*);
@@ -597,7 +644,7 @@ void HTMLDocumentNode_SecMgr_Init(HTMLDocumentNode*);
 
 HRESULT HTMLCurrentStyle_Create(HTMLElement*,IHTMLCurrentStyle**);
 
-void ConnectionPoint_Init(ConnectionPoint*,ConnectionPointContainer*,REFIID);
+void ConnectionPoint_Init(ConnectionPoint*,ConnectionPointContainer*,REFIID,cp_static_data_t*);
 void ConnectionPointContainer_Init(ConnectionPointContainer*,IUnknown*);
 void ConnectionPointContainer_Destroy(ConnectionPointContainer*);
 
@@ -625,9 +672,11 @@ void release_nsio(void);
 BOOL install_wine_gecko(BOOL);
 
 HRESULT nsuri_to_url(LPCWSTR,BOOL,BSTR*);
+HRESULT create_doc_uri(HTMLWindow*,WCHAR*,nsIWineURI**);
 
 HRESULT hlink_frame_navigate(HTMLDocument*,LPCWSTR,nsIInputStream*,DWORD);
-HRESULT navigate_url(HTMLDocumentNode*,OLECHAR*);
+HRESULT navigate_url(HTMLWindow*,const WCHAR*,const WCHAR*);
+HRESULT set_frame_doc(HTMLFrameBase*,nsIDOMDocument*);
 
 void call_property_onchanged(ConnectionPoint*,DISPID);
 HRESULT call_set_active_object(IOleInPlaceUIWindow*,IOleInPlaceActiveObject*);
@@ -635,10 +684,8 @@ HRESULT call_set_active_object(IOleInPlaceUIWindow*,IOleInPlaceActiveObject*);
 void *nsalloc(size_t) __WINE_ALLOC_SIZE(1);
 void nsfree(void*);
 
-void nsACString_Init(nsACString*,const char*);
 void nsACString_SetData(nsACString*,const char*);
 PRUint32 nsACString_GetData(const nsACString*,const char**);
-void nsACString_Finish(nsACString*);
 
 void nsAString_Init(nsAString*,const PRUnichar*);
 void nsAString_SetData(nsAString*,const PRUnichar*);
@@ -648,21 +695,26 @@ void nsAString_Finish(nsAString*);
 nsICommandParams *create_nscommand_params(void);
 HRESULT nsnode_to_nsstring(nsIDOMNode*,nsAString*);
 void get_editor_controller(NSContainer*);
-void init_nsevents(NSContainer*);
-void add_nsevent_listener(HTMLWindow*,LPCWSTR);
 nsresult get_nsinterface(nsISupports*,REFIID,void**);
 
-void set_document_bscallback(HTMLDocument*,nsChannelBSC*);
-void set_current_mon(HTMLDocument*,IMoniker*);
-HRESULT start_binding(HTMLDocument*,BSCallback*,IBindCtx*);
+void init_nsevents(HTMLDocumentNode*);
+void release_nsevents(HTMLDocumentNode*);
+void add_nsevent_listener(HTMLDocumentNode*,LPCWSTR);
 
-HRESULT bind_mon_to_buffer(HTMLDocument*,IMoniker*,void**,DWORD*);
+void set_window_bscallback(HTMLWindow*,nsChannelBSC*);
+void set_current_mon(HTMLWindow*,IMoniker*);
+HRESULT start_binding(HTMLWindow*,HTMLDocumentNode*,BSCallback*,IBindCtx*);
+void abort_document_bindings(HTMLDocumentNode*);
+
+HRESULT bind_mon_to_buffer(HTMLDocumentNode*,IMoniker*,void**,DWORD*);
 
 nsChannelBSC *create_channelbsc(IMoniker*);
 HRESULT channelbsc_load_stream(nsChannelBSC*,IStream*);
 void channelbsc_set_channel(nsChannelBSC*,nsChannel*,nsIStreamListener*,nsISupports*);
 IMoniker *get_channelbsc_mon(nsChannelBSC*);
 
+void set_ready_state(HTMLWindow*,READYSTATE);
+
 HRESULT HTMLSelectionObject_Create(HTMLDocumentNode*,nsISelection*,IHTMLSelectionObject**);
 HRESULT HTMLTxtRange_Create(HTMLDocumentNode*,nsIDOMRange*,IHTMLTxtRange**);
 IHTMLStyle *HTMLStyle_Create(nsIDOMCSSStyleDeclaration*);
@@ -679,24 +731,27 @@ HTMLDOMNode *HTMLDOMTextNode_Create(HTMLDocumentNode*,nsIDOMNode*);
 
 HTMLElement *HTMLElement_Create(HTMLDocumentNode*,nsIDOMNode*,BOOL);
 HTMLElement *HTMLCommentElement_Create(HTMLDocumentNode*,nsIDOMNode*);
-HTMLElement *HTMLAnchorElement_Create(nsIDOMHTMLElement*);
-HTMLElement *HTMLBodyElement_Create(nsIDOMHTMLElement*);
-HTMLElement *HTMLIFrame_Create(nsIDOMHTMLElement*);
-HTMLElement *HTMLImgElement_Create(nsIDOMHTMLElement*);
-HTMLElement *HTMLInputElement_Create(nsIDOMHTMLElement*);
-HTMLElement *HTMLOptionElement_Create(nsIDOMHTMLElement*);
-HTMLElement *HTMLScriptElement_Create(nsIDOMHTMLElement*);
-HTMLElement *HTMLSelectElement_Create(nsIDOMHTMLElement*);
-HTMLElement *HTMLTable_Create(nsIDOMHTMLElement*);
-HTMLElement *HTMLTableRow_Create(nsIDOMHTMLElement*);
-HTMLElement *HTMLTextAreaElement_Create(nsIDOMHTMLElement*);
-HTMLElement *HTMLGenericElement_Create(nsIDOMHTMLElement*);
+HTMLElement *HTMLAnchorElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HTMLElement *HTMLBodyElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HTMLElement *HTMLFormElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HTMLElement *HTMLFrameElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HTMLElement *HTMLIFrame_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HTMLElement *HTMLImgElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HTMLElement *HTMLInputElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HTMLElement *HTMLOptionElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HTMLElement *HTMLScriptElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HTMLElement *HTMLSelectElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HTMLElement *HTMLTable_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HTMLElement *HTMLTableRow_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HTMLElement *HTMLTextAreaElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HTMLElement *HTMLGenericElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
 
 void HTMLDOMNode_Init(HTMLDocumentNode*,HTMLDOMNode*,nsIDOMNode*);
-void HTMLElement_Init(HTMLElement*,dispex_static_data_t*);
+void HTMLElement_Init(HTMLElement*,HTMLDocumentNode*,nsIDOMHTMLElement*,dispex_static_data_t*);
 void HTMLElement2_Init(HTMLElement*);
 void HTMLElement3_Init(HTMLElement*);
-void HTMLTextContainer_Init(HTMLTextContainer*,dispex_static_data_t*);
+void HTMLTextContainer_Init(HTMLTextContainer*,HTMLDocumentNode*,nsIDOMHTMLElement*,dispex_static_data_t*);
+void HTMLFrameBase_Init(HTMLFrameBase*,HTMLDocumentNode*,nsIDOMHTMLElement*,dispex_static_data_t*);
 
 HRESULT HTMLDOMNode_QI(HTMLDOMNode*,REFIID,void**);
 void HTMLDOMNode_destructor(HTMLDOMNode*);
@@ -704,6 +759,9 @@ void HTMLDOMNode_destructor(HTMLDOMNode*);
 HRESULT HTMLElement_QI(HTMLDOMNode*,REFIID,void**);
 void HTMLElement_destructor(HTMLDOMNode*);
 
+HRESULT HTMLFrameBase_QI(HTMLFrameBase*,REFIID,void**);
+void HTMLFrameBase_destructor(HTMLFrameBase*);
+
 HTMLDOMNode *get_node(HTMLDocumentNode*,nsIDOMNode*,BOOL);
 void release_nodes(HTMLDocumentNode*);
 
@@ -750,20 +808,19 @@ void set_dirty(HTMLDocument*,VARIANT_BOOL);
 
 extern DWORD mshtml_tls;
 
-typedef struct task_t {
-    HTMLDocument *doc;
-
-    enum {
-        TASK_SETDOWNLOADSTATE,
-        TASK_PARSECOMPLETE,
-        TASK_SETPROGRESS,
-        TASK_START_BINDING
-    } task_id;
-
-    nsChannelBSC *bscallback;
+typedef struct task_t task_t;
+typedef void (*task_proc_t)(task_t*);
 
+struct task_t {
+    LONG target_magic;
+    task_proc_t proc;
     struct task_t *next;
-} task_t;
+};
+
+typedef struct {
+    task_t header;
+    HTMLDocumentObj *doc;
+} docobj_task_t;
 
 typedef struct {
     HWND thread_hwnd;
@@ -774,8 +831,11 @@ typedef struct {
 
 thread_data_t *get_thread_data(BOOL);
 HWND get_thread_hwnd(void);
-void push_task(task_t*);
-void remove_doc_tasks(const HTMLDocument*);
+
+LONG get_task_target_magic(void);
+void push_task(task_t*,task_proc_t,LONG);
+void remove_target_tasks(LONG);
+
 DWORD set_task_timer(HTMLDocument*,DWORD,BOOL,IDispatch*);
 HRESULT clear_task_timer(HTMLDocument*,BOOL,DWORD);
 
@@ -877,6 +937,18 @@ static inline char *heap_strdupWtoA(LPCWSTR str)
     return ret;
 }
 
+static inline void windowref_addref(windowref_t *ref)
+{
+    InterlockedIncrement(&ref->ref);
+}
+
+static inline void windowref_release(windowref_t *ref)
+{
+    if(!InterlockedDecrement(&ref->ref))
+        heap_free(ref);
+}
+
+HDC get_display_dc(void);
 HINSTANCE get_shdoclc(void);
 
 extern HINSTANCE hInst;
