@@ -27,9 +27,12 @@
 #include "windef.h"
 #include "winbase.h"
 #include "winuser.h"
+#include "winreg.h"
 #include "ole2.h"
 #include "hlguids.h"
 #include "shlguid.h"
+#include "wininet.h"
+#include "shlwapi.h"
 
 #include "wine/debug.h"
 #include "wine/unicode.h"
@@ -41,8 +44,6 @@ WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 #define CONTENT_LENGTH "Content-Length"
 #define UTF16_STR "utf-16"
 
-static WCHAR emptyW[] = {0};
-
 typedef struct {
     const nsIInputStreamVtbl *lpInputStreamVtbl;
 
@@ -84,7 +85,7 @@ struct BSCallback {
     IMoniker *mon;
     IBinding *binding;
 
-    HTMLDocument *doc;
+    HTMLDocumentNode *doc;
 
     struct list entry;
 };
@@ -315,7 +316,7 @@ static HRESULT WINAPI BindStatusCallback_OnStartBinding(IBindStatusCallback *ifa
     This->binding = pbind;
 
     if(This->doc)
-        list_add_head(&This->doc->doc_obj->bindings, &This->entry);
+        list_add_head(&This->doc->bindings, &This->entry);
 
     return This->vtbl->start_binding(This);
 }
@@ -707,7 +708,7 @@ static void parse_post_data(nsIInputStream *post_data_stream, LPWSTR *headers_re
     *post_data_len_ret = post_data_len;
 }
 
-HRESULT start_binding(HTMLDocument *doc, BSCallback *bscallback, IBindCtx *bctx)
+HRESULT start_binding(HTMLWindow *window, HTMLDocumentNode *doc, BSCallback *bscallback, IBindCtx *bctx)
 {
     IStream *str = NULL;
     HRESULT hres;
@@ -716,7 +717,8 @@ HRESULT start_binding(HTMLDocument *doc, BSCallback *bscallback, IBindCtx *bctx)
 
     /* NOTE: IE7 calls IsSystemMoniker here*/
 
-    call_docview_84(doc->doc_obj);
+    if(window)
+        call_docview_84(window->doc_obj);
 
     if(bctx) {
         RegisterBindStatusCallback(bctx, STATUSCLB(bscallback), NULL, 0);
@@ -848,14 +850,14 @@ static BufferBSC *create_bufferbsc(IMoniker *mon)
     return ret;
 }
 
-HRESULT bind_mon_to_buffer(HTMLDocument *doc, IMoniker *mon, void **buf, DWORD *size)
+HRESULT bind_mon_to_buffer(HTMLDocumentNode *doc, IMoniker *mon, void **buf, DWORD *size)
 {
     BufferBSC *bsc = create_bufferbsc(mon);
     HRESULT hres;
 
     *buf = NULL;
 
-    hres = start_binding(doc, &bsc->bsc, NULL);
+    hres = start_binding(NULL, doc, &bsc->bsc, NULL);
     if(SUCCEEDED(hres)) {
         hres = bsc->hres;
         if(SUCCEEDED(hres)) {
@@ -874,6 +876,8 @@ HRESULT bind_mon_to_buffer(HTMLDocument *doc, IMoniker *mon, void **buf, DWORD *
 struct nsChannelBSC {
     BSCallback bsc;
 
+    HTMLWindow *window;
+
     nsChannel *nschannel;
     nsIStreamListener *nslistener;
     nsISupports *nscontext;
@@ -950,11 +954,8 @@ static HRESULT read_stream_data(nsChannelBSC *This, IStream *stream)
 
             on_start_nsrequest(This);
 
-            /* events are reset when a new document URI is loaded, so re-initialise them here */
-            if(This->bsc.doc && This->bsc.doc->doc_obj->bscallback == This && This->bsc.doc->doc_obj->nscontainer) {
-                update_window_doc(This->bsc.doc->window);
-                init_nsevents(This->bsc.doc->doc_obj->nscontainer);
-            }
+            if(This->window)
+                update_window_doc(This->window);
         }
 
         This->bsc.readed += This->nsstream->buf_size;
@@ -1107,41 +1108,52 @@ IMoniker *get_channelbsc_mon(nsChannelBSC *This)
     return This->bsc.mon;
 }
 
-void set_document_bscallback(HTMLDocument *doc, nsChannelBSC *callback)
+void set_window_bscallback(HTMLWindow *window, nsChannelBSC *callback)
+{
+    if(window->bscallback) {
+        if(window->bscallback->bsc.binding)
+            IBinding_Abort(window->bscallback->bsc.binding);
+        window->bscallback->bsc.doc = NULL;
+        window->bscallback->window = NULL;
+        IBindStatusCallback_Release(STATUSCLB(&window->bscallback->bsc));
+    }
+
+    window->bscallback = callback;
+
+    if(callback) {
+        callback->window = window;
+        IBindStatusCallback_AddRef(STATUSCLB(&callback->bsc));
+        callback->bsc.doc = window->doc;
+    }
+}
+
+void abort_document_bindings(HTMLDocumentNode *doc)
 {
     BSCallback *iter;
 
-    if(doc->doc_obj->bscallback) {
-        if(doc->doc_obj->bscallback->bsc.binding)
-            IBinding_Abort(doc->doc_obj->bscallback->bsc.binding);
-        doc->doc_obj->bscallback->bsc.doc = NULL;
-        IBindStatusCallback_Release(STATUSCLB(&doc->doc_obj->bscallback->bsc));
-    }
-
-    LIST_FOR_EACH_ENTRY(iter, &doc->doc_obj->bindings, BSCallback, entry) {
+    LIST_FOR_EACH_ENTRY(iter, &doc->bindings, BSCallback, entry) {
+        if(iter->binding)
+            IBinding_Abort(iter->binding);
         iter->doc = NULL;
         list_remove(&iter->entry);
     }
-
-    doc->doc_obj->bscallback = callback;
-
-    if(callback) {
-        IBindStatusCallback_AddRef(STATUSCLB(&callback->bsc));
-        callback->bsc.doc = doc;
-    }
 }
 
 HRESULT channelbsc_load_stream(nsChannelBSC *bscallback, IStream *stream)
 {
     HRESULT hres;
 
-    const char text_html[] = "text/html";
+    if(!bscallback->nschannel) {
+        ERR("NULL nschannel\n");
+        return E_FAIL;
+    }
+
+    bscallback->nschannel->content_type = heap_strdupA("text/html");
+    if(!bscallback->nschannel->content_type)
+        return E_OUTOFMEMORY;
 
     add_nsrequest(bscallback);
 
-    if(bscallback->nschannel)
-        bscallback->nschannel->content_type = heap_strdupA(text_html);
-
     hres = read_stream_data(bscallback, stream);
     IBindStatusCallback_OnStopBinding(STATUSCLB(&bscallback->bsc), hres, ERROR_SUCCESS);
 
@@ -1220,25 +1232,72 @@ HRESULT hlink_frame_navigate(HTMLDocument *doc, LPCWSTR url,
     return hres;
 }
 
-HRESULT navigate_url(HTMLDocumentNode *doc, OLECHAR *url)
+
+HRESULT navigate_url(HTMLWindow *window, const WCHAR *new_url, const WCHAR *base_url)
 {
-    OLECHAR *translated_url = NULL;
+    WCHAR url[INTERNET_MAX_URL_LENGTH];
+    nsIWebNavigation *web_navigation;
+    nsIDocShell *doc_shell;
+    nsIWineURI *uri;
+    nsresult nsres;
     HRESULT hres;
 
-    if(!url)
-        url = emptyW;
+    if(!new_url) {
+        *url = 0;
+    }else if(base_url) {
+        DWORD len = 0;
 
-    if(doc->basedoc.doc_obj->hostui) {
-        hres = IDocHostUIHandler_TranslateUrl(doc->basedoc.doc_obj->hostui, 0, url,
-                &translated_url);
-        if(hres == S_OK)
-            url = translated_url;
+        hres = CoInternetCombineUrl(base_url, new_url, URL_ESCAPE_SPACES_ONLY|URL_DONT_ESCAPE_EXTRA_INFO,
+                url, sizeof(url)/sizeof(WCHAR), &len, 0);
+        if(FAILED(hres))
+            return hres;
+    }else {
+        strcpyW(url, new_url);
     }
 
-    hres = hlink_frame_navigate(&doc->basedoc, url, NULL, 0);
-    if(FAILED(hres))
-        FIXME("hlink_frame_navigate failed: %08x\n", hres);
+    if(window->doc_obj && window->doc_obj->hostui) {
+        OLECHAR *translated_url = NULL;
 
-    CoTaskMemFree(translated_url);
-    return hres;
+        hres = IDocHostUIHandler_TranslateUrl(window->doc_obj->hostui, 0, url,
+                &translated_url);
+        if(hres == S_OK) {
+            strcpyW(url, translated_url);
+            CoTaskMemFree(translated_url);
+        }
+    }
+
+    if(window->doc_obj && window == window->doc_obj->basedoc.window) {
+        hres = hlink_frame_navigate(&window->doc->basedoc, url, NULL, 0);
+        if(SUCCEEDED(hres))
+            return S_OK;
+        TRACE("hlink_frame_navigate failed: %08x\n", hres);
+    }
+
+    nsres = get_nsinterface((nsISupports*)window->nswindow, &IID_nsIWebNavigation, (void**)&web_navigation);
+    if(NS_FAILED(nsres)) {
+        ERR("Could not get nsIWebNavigation interface: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    nsres = nsIWebNavigation_QueryInterface(web_navigation, &IID_nsIDocShell, (void**)&doc_shell);
+    nsIWebNavigation_Release(web_navigation);
+    if(NS_FAILED(nsres)) {
+        ERR("Could not get nsIDocShell: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    hres = create_doc_uri(window, url, &uri);
+    if(FAILED(hres)) {
+        nsIDocShell_Release(doc_shell);
+        return hres;
+    }
+
+    nsres = nsIDocShell_LoadURI(doc_shell, (nsIURI*)uri, NULL, 0, FALSE);
+    nsIDocShell_Release(doc_shell);
+    if(NS_FAILED(nsres)) {
+        WARN("LoadURI failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    return S_OK;
 }
