@@ -59,6 +59,7 @@ typedef struct {
     nsIURI *uri;
     nsIURL *nsurl;
     NSContainer *container;
+    windowref_t *window_ref;
     LPWSTR wine_url;
     PRBool is_doc_uri;
     BOOL use_wine_url;
@@ -67,7 +68,7 @@ typedef struct {
 #define NSURI(x)         ((nsIURI*)            &(x)->lpWineURIVtbl)
 #define NSWINEURI(x)     ((nsIWineURI*)        &(x)->lpWineURIVtbl)
 
-static nsresult create_uri(nsIURI*,NSContainer*,nsIWineURI**);
+static nsresult create_uri(nsIURI*,HTMLWindow*,NSContainer*,nsIWineURI**);
 
 static const char *debugstr_nsacstr(const nsACString *nsstr)
 {
@@ -157,6 +158,11 @@ static BOOL before_async_open(nsChannel *channel, NSContainer *container)
     return hres != S_OK;
 }
 
+static inline BOOL is_http_channel(nsChannel *This)
+{
+    return This->url_scheme == URL_SCHEME_HTTP || This->url_scheme == URL_SCHEME_HTTP;
+}
+
 #define NSCHANNEL_THIS(iface) DEFINE_THIS(nsChannel, HttpChannel, iface)
 
 static nsresult NSAPI nsChannel_QueryInterface(nsIHttpChannel *iface, nsIIDRef riid, nsQIResult result)
@@ -174,13 +180,13 @@ static nsresult NSAPI nsChannel_QueryInterface(nsIHttpChannel *iface, nsIIDRef r
         *result = NSCHANNEL(This);
     }else if(IsEqualGUID(&IID_nsIHttpChannel, riid)) {
         TRACE("(%p)->(IID_nsIHttpChannel %p)\n", This, result);
-        *result = This->http_channel ? NSHTTPCHANNEL(This) : NULL;
+        *result = is_http_channel(This) ? NSHTTPCHANNEL(This) : NULL;
     }else if(IsEqualGUID(&IID_nsIUploadChannel, riid)) {
         TRACE("(%p)->(IID_nsIUploadChannel %p)\n", This, result);
         *result = NSUPCHANNEL(This);
     }else if(IsEqualGUID(&IID_nsIHttpChannelInternal, riid)) {
         TRACE("(%p)->(IID_nsIHttpChannelInternal %p)\n", This, result);
-        *result = This->http_channel_internal ? NSHTTPINTERNAL(This) : NULL;
+        *result = is_http_channel(This) ? NSHTTPINTERNAL(This) : NULL;
     }else {
         TRACE("(%p)->(%s %p)\n", This, debugstr_guid(riid), result);
         *result = NULL;
@@ -635,9 +641,9 @@ static HRESULT create_mon_for_nschannel(nsChannel *channel, IMoniker **mon)
     return hres;
 }
 
-static NSContainer *get_nscontainer_from_load_group(nsChannel *This)
+static HTMLWindow *get_window_from_load_group(nsChannel *This)
 {
-    NSContainer *container;
+    HTMLWindow *window;
     nsIChannel *channel;
     nsIRequest *req;
     nsIWineURI *wine_uri;
@@ -674,79 +680,117 @@ static NSContainer *get_nscontainer_from_load_group(nsChannel *This)
         return NULL;
     }
 
-    nsIWineURI_GetNSContainer(wine_uri, &container);
+    nsIWineURI_GetWindow(wine_uri, &window);
     nsIWineURI_Release(wine_uri);
 
-    return container;
+    return window;
 }
 
-static nsresult async_open_doc_uri(nsChannel *This, NSContainer *container,
-        nsIStreamListener *listener, nsISupports *context, BOOL *open)
+static HTMLWindow *get_channel_window(nsChannel *This)
 {
-    IMoniker *mon;
-    HRESULT hres;
+    nsIRequestObserver *req_observer;
+    nsIWebProgress *web_progress;
+    nsIDOMWindow *nswindow;
+    HTMLWindow *window;
+    nsresult nsres;
 
-    *open = FALSE;
-
-    if(container->bscallback) {
-        channelbsc_set_channel(container->bscallback, This, listener, context);
-
-        if(container->doc && container->doc->mime) {
-            heap_free(This->content_type);
-            This->content_type = heap_strdupWtoA(container->doc->mime);
-        }
-
-        return NS_OK;
-    }else  {
-        BOOL cont = before_async_open(This, container);
-
-        if(!cont) {
-            TRACE("canceled\n");
-            return NS_ERROR_UNEXPECTED;
-        }
-
-        if(!container->doc) {
-            return This->channel
-                ?  nsIChannel_AsyncOpen(This->channel, listener, context)
-                : NS_ERROR_UNEXPECTED;
-        }
-
-        hres = create_mon_for_nschannel(This, &mon);
-        if(FAILED(hres)) {
-            return NS_ERROR_UNEXPECTED;
-        }
-        set_current_mon(&container->doc->basedoc, mon);
+    if(!This->load_group) {
+        ERR("NULL load_group\n");
+        return NULL;
     }
 
-    *open = TRUE;
-    return NS_OK;
+    nsres = nsILoadGroup_GetGroupObserver(This->load_group, &req_observer);
+    if(NS_FAILED(nsres) || !req_observer) {
+        ERR("GetGroupObserver failed: %08x\n", nsres);
+        return NULL;
+    }
+
+    nsres = nsIRequestObserver_QueryInterface(req_observer, &IID_nsIWebProgress, (void**)&web_progress);
+    nsIRequestObserver_Release(req_observer);
+    if(NS_FAILED(nsres)) {
+        ERR("Could not get nsIWebProgress iface: %08x\n", nsres);
+        return NULL;
+    }
+
+    nsres = nsIWebProgress_GetDOMWindow(web_progress, &nswindow);
+    nsIWebProgress_Release(web_progress);
+    if(NS_FAILED(nsres) || !nswindow) {
+        ERR("GetDOMWindow failed: %08x\n", nsres);
+        return NULL;
+    }
+
+    window = nswindow_to_window(nswindow);
+    nsIDOMWindow_Release(nswindow);
+
+    if(window)
+        IHTMLWindow2_AddRef(HTMLWINDOW2(window));
+    else
+        FIXME("NULL window for %p\n", nswindow);
+    return window;
 }
 
-static nsresult async_open(nsChannel *This, NSContainer *container, nsIStreamListener *listener,
+typedef struct {
+    task_t header;
+    HTMLDocumentNode *doc;
+    nsChannelBSC *bscallback;
+} start_binding_task_t;
+
+static void start_binding_proc(task_t *_task)
+{
+    start_binding_task_t *task = (start_binding_task_t*)_task;
+
+    start_binding(NULL, task->doc, (BSCallback*)task->bscallback, NULL);
+}
+
+typedef struct {
+    task_t header;
+    HTMLWindow *window;
+    nsChannelBSC *bscallback;
+} start_doc_binding_task_t;
+
+static void start_doc_binding_proc(task_t *_task)
+{
+    start_doc_binding_task_t *task = (start_doc_binding_task_t*)_task;
+
+    start_binding(task->window, NULL, (BSCallback*)task->bscallback, NULL);
+    IUnknown_Release((IUnknown*)task->bscallback);
+}
+
+static nsresult async_open(nsChannel *This, HTMLWindow *window, BOOL is_doc_channel, nsIStreamListener *listener,
         nsISupports *context)
 {
     nsChannelBSC *bscallback;
     IMoniker *mon = NULL;
-    task_t *task;
     HRESULT hres;
 
     hres = create_mon_for_nschannel(This, &mon);
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
+    if(is_doc_channel)
+        set_current_mon(window, mon);
+
     bscallback = create_channelbsc(mon);
     IMoniker_Release(mon);
 
     channelbsc_set_channel(bscallback, This, listener, context);
 
-    task = heap_alloc(sizeof(task_t));
+    if(is_doc_channel) {
+        start_doc_binding_task_t *task;
 
-    task->doc = &container->doc->basedoc;
-    task->task_id = TASK_START_BINDING;
-    task->next = NULL;
-    task->bscallback = bscallback;
+        set_window_bscallback(window, bscallback);
 
-    push_task(task);
+        task = heap_alloc(sizeof(start_doc_binding_task_t));
+        task->window = window;
+        task->bscallback = bscallback;
+        push_task(&task->header, start_doc_binding_proc, window->task_magic);
+    }else {
+        start_binding_task_t *task = heap_alloc(sizeof(start_binding_task_t));
+
+        task->doc = window->doc;
+        task->bscallback = bscallback;
+        push_task(&task->header, start_binding_proc, window->doc->basedoc.task_magic);
+    }
 
     return NS_OK;
 }
@@ -755,37 +799,88 @@ static nsresult NSAPI nsChannel_AsyncOpen(nsIHttpChannel *iface, nsIStreamListen
                                           nsISupports *aContext)
 {
     nsChannel *This = NSCHANNEL_THIS(iface);
-    NSContainer *container;
+    HTMLWindow *window = NULL;
     PRBool is_doc_uri;
     BOOL open = TRUE;
     nsresult nsres = NS_OK;
 
     TRACE("(%p)->(%p %p)\n", This, aListener, aContext);
 
-    nsIWineURI_GetNSContainer(This->uri, &container);
+    if(TRACE_ON(mshtml)) {
+        LPCWSTR url;
 
-    if(!container && This->load_group) {
-        container = get_nscontainer_from_load_group(This);
-        if(container)
-            nsIWineURI_SetNSContainer(This->uri, container);
+        nsIWineURI_GetWineURL(This->uri, &url);
+        TRACE("opening %s\n", debugstr_w(url));
     }
 
-    if(!container) {
-        TRACE("container = NULL\n");
+    nsIWineURI_GetIsDocumentURI(This->uri, &is_doc_uri);
+    if(is_doc_uri) {
+        window = get_channel_window(This);
+        if(window) {
+            nsIWineURI_SetWindow(This->uri, window);
+        }else {
+            NSContainer *nscontainer;
+
+            nsIWineURI_GetNSContainer(This->uri, &nscontainer);
+            if(nscontainer) {
+                BOOL b;
+
+                /* nscontainer->doc should be NULL which means navigation to a new window */
+                if(nscontainer->doc)
+                    FIXME("nscontainer->doc = %p\n", nscontainer->doc);
+
+                b = before_async_open(This, nscontainer);
+                nsIWebBrowserChrome_Release(NSWBCHROME(nscontainer));
+                if(b)
+                    FIXME("Navigation not cancelled\n");
+                return NS_ERROR_UNEXPECTED;
+            }
+        }
+    }
+
+    if(!window) {
+        nsIWineURI_GetWindow(This->uri, &window);
+
+        if(!window && This->load_group) {
+            window = get_window_from_load_group(This);
+            if(window)
+                nsIWineURI_SetWindow(This->uri, window);
+        }
+    }
+
+    if(!window) {
+        ERR("window = NULL\n");
         return This->channel
             ? nsIChannel_AsyncOpen(This->channel, aListener, aContext)
             : NS_ERROR_UNEXPECTED;
     }
 
-    nsIWineURI_GetIsDocumentURI(This->uri, &is_doc_uri);
+    if(is_doc_uri && (This->load_flags & LOAD_INITIAL_DOCUMENT_URI) && window == window->doc_obj->basedoc.window) {
+        if(window->doc_obj->nscontainer->bscallback) {
+            NSContainer *nscontainer = window->doc_obj->nscontainer;
 
-    if(is_doc_uri && (This->load_flags & LOAD_INITIAL_DOCUMENT_URI))
-        nsres = async_open_doc_uri(This, container, aListener, aContext, &open);
+            channelbsc_set_channel(nscontainer->bscallback, This, aListener, aContext);
+
+            if(nscontainer->doc->mime) {
+                heap_free(This->content_type);
+                This->content_type = heap_strdupWtoA(nscontainer->doc->mime);
+            }
+
+            open = FALSE;
+        }else {
+            open = before_async_open(This, window->doc_obj->nscontainer);
+            if(!open) {
+                TRACE("canceled\n");
+                nsres = NS_ERROR_UNEXPECTED;
+            }
+        }
+    }
 
     if(open)
-        nsres = async_open(This, container, aListener, aContext);
+        nsres = async_open(This, window, is_doc_uri, aListener, aContext);
 
-    nsIWebBrowserChrome_Release(NSWBCHROME(container));
+    if(window)
+        IHTMLWindow2_Release(HTMLWINDOW2(window));
     return nsres;
 }
 
@@ -1347,6 +1442,8 @@ static nsrefcnt NSAPI nsURI_Release(nsIWineURI *iface)
     TRACE("(%p) ref=%d\n", This, ref);
 
     if(!ref) {
+        if(This->window_ref)
+            windowref_release(This->window_ref);
         if(This->container)
             nsIWebBrowserChrome_Release(NSWBCHROME(This->container));
         if(This->nsurl)
@@ -1414,7 +1511,7 @@ static nsresult NSAPI nsURI_GetScheme(nsIWineURI *iface, nsACString *aScheme)
 
     TRACE("(%p)->(%p)\n", This, aScheme);
 
-    if(This->use_wine_url && strcmpW(This->wine_url, about_blankW)) {
+    if(This->use_wine_url) {
         /*
          * For Gecko we set scheme to unknown so it won't be handled
          * as any special case.
@@ -1662,7 +1759,7 @@ static nsresult NSAPI nsURI_Equals(nsIWineURI *iface, nsIURI *other, PRBool *_re
     }
 
     nsIWineURI_GetWineURL(wine_uri, &other_url);
-    *_retval = !UrlCompareW(This->wine_url, other_url, TRUE);
+    *_retval = other_url && !UrlCompareW(This->wine_url, other_url, TRUE);
     nsIWineURI_Release(wine_uri);
 
     return NS_OK;
@@ -1708,7 +1805,7 @@ static nsresult NSAPI nsURI_Clone(nsIWineURI *iface, nsIURI **_retval)
         }
     }
 
-    nsres = create_uri(nsuri, This->container, &wine_uri);
+    nsres = create_uri(nsuri, This->window_ref ? This->window_ref->window : NULL, This->container, &wine_uri);
     if(NS_FAILED(nsres)) {
         WARN("create_uri failed: %08x\n", nsres);
         return nsres;
@@ -1908,12 +2005,17 @@ static nsresult NSAPI nsURL_GetRef(nsIWineURI *iface, nsACString *aRef)
 static nsresult NSAPI nsURL_SetRef(nsIWineURI *iface, const nsACString *aRef)
 {
     nsURI *This = NSURI_THIS(iface);
+    const char *refa;
 
     TRACE("(%p)->(%s)\n", This, debugstr_nsacstr(aRef));
 
     if(This->nsurl)
         return nsIURL_SetRef(This->nsurl, aRef);
 
+    nsACString_GetData(aRef, &refa);
+    if(!*refa)
+        return NS_OK;
+
     FIXME("default action not implemented\n");
     return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -2081,6 +2183,48 @@ static nsresult NSAPI nsURI_SetNSContainer(nsIWineURI *iface, NSContainer *aCont
     return NS_OK;
 }
 
+static nsresult NSAPI nsURI_GetWindow(nsIWineURI *iface, HTMLWindow **aHTMLWindow)
+{
+    nsURI *This = NSURI_THIS(iface);
+
+    TRACE("(%p)->(%p)\n", This, aHTMLWindow);
+
+    if(This->window_ref && This->window_ref->window) {
+        IHTMLWindow2_AddRef(HTMLWINDOW2(This->window_ref->window));
+        *aHTMLWindow = This->window_ref->window;
+    }else {
+        *aHTMLWindow = NULL;
+    }
+
+    return NS_OK;
+}
+
+static nsresult NSAPI nsURI_SetWindow(nsIWineURI *iface, HTMLWindow *aHTMLWindow)
+{
+    nsURI *This = NSURI_THIS(iface);
+
+    TRACE("(%p)->(%p)\n", This, aHTMLWindow);
+
+    if(This->window_ref) {
+        if(This->window_ref->window == aHTMLWindow)
+            return NS_OK;
+        TRACE("Changing %p -> %p\n", This->window_ref->window, aHTMLWindow);
+        windowref_release(This->window_ref);
+    }
+
+    if(aHTMLWindow) {
+        windowref_addref(aHTMLWindow->window_ref);
+        This->window_ref = aHTMLWindow->window_ref;
+
+        if(aHTMLWindow->doc_obj)
+            nsIWineURI_SetNSContainer(NSWINEURI(This), aHTMLWindow->doc_obj->nscontainer);
+    }else {
+        This->window_ref = NULL;
+    }
+
+    return NS_OK;
+}
+
 static nsresult NSAPI nsURI_GetIsDocumentURI(nsIWineURI *iface, PRBool *aIsDocumentURI)
 {
     nsURI *This = NSURI_THIS(iface);
@@ -2128,11 +2272,15 @@ static nsresult NSAPI nsURI_SetWineURL(nsIWineURI *iface, LPCWSTR aURL)
         This->wine_url = heap_alloc(len*sizeof(WCHAR));
         memcpy(This->wine_url, aURL, len*sizeof(WCHAR));
 
-        /* FIXME: Always use wine url */
-        This->use_wine_url =
-               strncmpW(aURL, wszFtp,   sizeof(wszFtp)/sizeof(WCHAR))
-            && strncmpW(aURL, wszHttp,  sizeof(wszHttp)/sizeof(WCHAR))
-            && strncmpW(aURL, wszHttps, sizeof(wszHttps)/sizeof(WCHAR));
+        if(This->uri) {
+            /* FIXME: Always use wine url */
+            This->use_wine_url =
+                   strncmpW(aURL, wszFtp,   sizeof(wszFtp)/sizeof(WCHAR))
+                && strncmpW(aURL, wszHttp,  sizeof(wszHttp)/sizeof(WCHAR))
+                && strncmpW(aURL, wszHttps, sizeof(wszHttps)/sizeof(WCHAR));
+        }else {
+            This->use_wine_url = TRUE;
+        }
     }else {
         This->wine_url = NULL;
         This->use_wine_url = FALSE;
@@ -2193,26 +2341,24 @@ static const nsIWineURIVtbl nsWineURIVtbl = {
     nsURL_GetRelativeSpec,
     nsURI_GetNSContainer,
     nsURI_SetNSContainer,
+    nsURI_GetWindow,
+    nsURI_SetWindow,
     nsURI_GetIsDocumentURI,
     nsURI_SetIsDocumentURI,
     nsURI_GetWineURL,
     nsURI_SetWineURL
 };
 
-static nsresult create_uri(nsIURI *uri, NSContainer *container, nsIWineURI **_retval)
+static nsresult create_uri(nsIURI *uri, HTMLWindow *window, NSContainer *container, nsIWineURI **_retval)
 {
-    nsURI *ret = heap_alloc(sizeof(nsURI));
+    nsURI *ret = heap_alloc_zero(sizeof(nsURI));
 
     ret->lpWineURIVtbl = &nsWineURIVtbl;
     ret->ref = 1;
     ret->uri = uri;
-    ret->container = container;
-    ret->wine_url = NULL;
-    ret->is_doc_uri = FALSE;
-    ret->use_wine_url = FALSE;
 
-    if(container)
-        nsIWebBrowserChrome_AddRef(NSWBCHROME(container));
+    nsIWineURI_SetNSContainer(NSWINEURI(ret), container);
+    nsIWineURI_SetWindow(NSWINEURI(ret), window);
 
     if(uri)
         nsIURI_QueryInterface(uri, &IID_nsIURL, (void**)&ret->nsurl);
@@ -2224,6 +2370,22 @@ static nsresult create_uri(nsIURI *uri, NSContainer *container, nsIWineURI **_re
     return NS_OK;
 }
 
+HRESULT create_doc_uri(HTMLWindow *window, WCHAR *url, nsIWineURI **ret)
+{
+    nsIWineURI *uri;
+    nsresult nsres;
+
+    nsres = create_uri(NULL, window, window->doc_obj->nscontainer, &uri);
+    if(NS_FAILED(nsres))
+        return E_FAIL;
+
+    nsIWineURI_SetWineURL(uri, url);
+    nsIWineURI_SetIsDocumentURI(uri, TRUE);
+
+    *ret = uri;
+    return S_OK;
+}
+
 typedef struct {
     const nsIProtocolHandlerVtbl  *lpProtocolHandlerVtbl;
 
@@ -2449,7 +2611,7 @@ static nsresult NSAPI nsIOService_NewURI(nsIIOService *iface, const nsACString *
         const char *aOriginCharset, nsIURI *aBaseURI, nsIURI **_retval)
 {
     const char *spec = NULL;
-    NSContainer *nscontainer = NULL;
+    HTMLWindow *window = NULL;
     nsIURI *uri = NULL;
     LPCWSTR base_wine_url = NULL;
     nsIWineURI *base_wine_uri = NULL, *wine_uri;
@@ -2470,43 +2632,30 @@ static nsresult NSAPI nsIOService_NewURI(nsIIOService *iface, const nsACString *
     }
 
     if(aBaseURI) {
-        nsACString base_uri_str;
-        const char *base_uri = NULL;
+        PARSEDURLA parsed_url = {sizeof(PARSEDURLA)};
 
-        nsACString_Init(&base_uri_str, NULL);
-
-        nsres = nsIURI_GetSpec(aBaseURI, &base_uri_str);
+        nsres = nsIURI_QueryInterface(aBaseURI, &IID_nsIWineURI, (void**)&base_wine_uri);
         if(NS_SUCCEEDED(nsres)) {
-            nsACString_GetData(&base_uri_str, &base_uri);
-            TRACE("base_uri=%s\n", debugstr_a(base_uri));
+            nsIWineURI_GetWineURL(base_wine_uri, &base_wine_url);
+            nsIWineURI_GetWindow(base_wine_uri, &window);
+            TRACE("base url: %s window: %p\n", debugstr_w(base_wine_url), window);
+        }else if(FAILED(ParseURLA(spec, &parsed_url))) {
+            TRACE("not wraping\n");
+            return nsIIOService_NewURI(nsio, aSpec, aOriginCharset, aBaseURI, _retval);
         }else {
-            ERR("GetSpec failed: %08x\n", nsres);
+            WARN("Could not get base nsIWineURI: %08x\n", nsres);
         }
-
-        nsACString_Finish(&base_uri_str);
     }
 
     nsres = nsIIOService_NewURI(nsio, aSpec, aOriginCharset, aBaseURI, &uri);
     if(NS_FAILED(nsres))
         TRACE("NewURI failed: %08x\n", nsres);
 
-    if(aBaseURI) {
-        nsres = nsIURI_QueryInterface(aBaseURI, &IID_nsIWineURI, (void**)&base_wine_uri);
-        if(NS_SUCCEEDED(nsres)) {
-            nsIWineURI_GetNSContainer(base_wine_uri, &nscontainer);
-            nsIWineURI_GetWineURL(base_wine_uri, &base_wine_url);
-        }else {
-            TRACE("Could not get base nsIWineURI: %08x\n", nsres);
-        }
-    }
-
-    TRACE("nscontainer = %p\n", nscontainer);
-
-    nsres = create_uri(uri, nscontainer, &wine_uri);
+    nsres = create_uri(uri, window, NULL, &wine_uri);
     *_retval = (nsIURI*)wine_uri;
 
-    if(nscontainer)
-        nsIWebBrowserChrome_Release(NSWBCHROME(nscontainer));
+    if(window)
+        IHTMLWindow2_Release(HTMLWINDOW2(window));
 
     if(base_wine_url) {
         WCHAR url[INTERNET_MAX_URL_LENGTH], rel_url[INTERNET_MAX_URL_LENGTH];
@@ -2545,27 +2694,23 @@ static nsresult NSAPI nsIOService_NewFileURI(nsIIOService *iface, nsIFile *aFile
 static nsresult NSAPI nsIOService_NewChannelFromURI(nsIIOService *iface, nsIURI *aURI,
                                                      nsIChannel **_retval)
 {
+    PARSEDURLW parsed_url = {sizeof(PARSEDURLW)};
     nsIChannel *channel = NULL;
     nsChannel *ret;
     nsIWineURI *wine_uri;
+    const WCHAR *url;
     nsresult nsres;
 
     TRACE("(%p %p)\n", aURI, _retval);
 
-    nsres = nsIIOService_NewChannelFromURI(nsio, aURI, &channel);
-    if(NS_FAILED(nsres) && nsres != NS_ERROR_UNKNOWN_PROTOCOL) {
-        WARN("NewChannelFromURI failed: %08x\n", nsres);
-        *_retval = channel;
-        return nsres;
-    }
-
     nsres = nsIURI_QueryInterface(aURI, &IID_nsIWineURI, (void**)&wine_uri);
     if(NS_FAILED(nsres)) {
-        WARN("Could not get nsIWineURI: %08x\n", nsres);
-        *_retval = channel;
-        return channel ? NS_OK : NS_ERROR_UNEXPECTED;
+        TRACE("Could not get nsIWineURI: %08x\n", nsres);
+        return nsIIOService_NewChannelFromURI(nsio, aURI, _retval);
     }
 
+    nsIIOService_NewChannelFromURI(nsio, aURI, &channel);
+
     ret = heap_alloc_zero(sizeof(nsChannel));
 
     ret->lpHttpChannelVtbl = &nsChannelVtbl;
@@ -2578,6 +2723,9 @@ static nsresult NSAPI nsIOService_NewChannelFromURI(nsIIOService *iface, nsIURI
     nsIURI_AddRef(aURI);
     ret->original_uri = aURI;
 
+    nsIWineURI_GetWineURL(wine_uri, &url);
+    ret->url_scheme = url && SUCCEEDED(ParseURLW(url, &parsed_url)) ? parsed_url.nScheme : URL_SCHEME_UNKNOWN;
+
     if(channel) {
         nsIChannel_QueryInterface(channel, &IID_nsIHttpChannel, (void**)&ret->http_channel);
         nsIChannel_QueryInterface(channel, &IID_nsIHttpChannelInternal, (void**)&ret->http_channel_internal);
