@@ -36,6 +36,7 @@
 #include "wine/unicode.h"
 
 #include "mshtml_private.h"
+#include "htmlevent.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
@@ -48,38 +49,115 @@ static BOOL use_gecko_script(LPCWSTR url)
         && strncmpiW(aboutW, url, sizeof(aboutW)/sizeof(WCHAR));
 }
 
-void set_current_mon(HTMLDocument *This, IMoniker *mon)
+void set_current_mon(HTMLWindow *This, IMoniker *mon)
 {
     HRESULT hres;
 
-    if(This->doc_obj->mon) {
-        IMoniker_Release(This->doc_obj->mon);
-        This->doc_obj->mon = NULL;
+    if(This->mon) {
+        IMoniker_Release(This->mon);
+        This->mon = NULL;
     }
 
-    if(This->doc_obj->url) {
-        CoTaskMemFree(This->doc_obj->url);
-        This->doc_obj->url = NULL;
+    if(This->url) {
+        CoTaskMemFree(This->url);
+        This->url = NULL;
     }
 
     if(!mon)
         return;
 
     IMoniker_AddRef(mon);
-    This->doc_obj->mon = mon;
+    This->mon = mon;
 
-    hres = IMoniker_GetDisplayName(mon, NULL, NULL, &This->doc_obj->url);
+    hres = IMoniker_GetDisplayName(mon, NULL, NULL, &This->url);
     if(FAILED(hres))
         WARN("GetDisplayName failed: %08x\n", hres);
 
-    set_script_mode(This->window, use_gecko_script(This->doc_obj->url) ? SCRIPTMODE_GECKO : SCRIPTMODE_ACTIVESCRIPT);
+    set_script_mode(This, use_gecko_script(This->url) ? SCRIPTMODE_GECKO : SCRIPTMODE_ACTIVESCRIPT);
 }
 
-static HRESULT set_moniker(HTMLDocument *This, IMoniker *mon, IBindCtx *pibc, BOOL *bind_complete)
+static void set_progress_proc(task_t *_task)
+{
+    docobj_task_t *task = (docobj_task_t*)_task;
+    IOleCommandTarget *olecmd = NULL;
+    HTMLDocumentObj *doc = task->doc;
+    HRESULT hres;
+
+    TRACE("(%p)\n", doc);
+
+    if(doc->client)
+        IOleClientSite_QueryInterface(doc->client, &IID_IOleCommandTarget, (void**)&olecmd);
+
+    if(olecmd) {
+        VARIANT progress_max, progress;
+
+        V_VT(&progress_max) = VT_I4;
+        V_I4(&progress_max) = 0; /* FIXME */
+        IOleCommandTarget_Exec(olecmd, NULL, OLECMDID_SETPROGRESSMAX, OLECMDEXECOPT_DONTPROMPTUSER,
+                               &progress_max, NULL);
+
+        V_VT(&progress) = VT_I4;
+        V_I4(&progress) = 0; /* FIXME */
+        IOleCommandTarget_Exec(olecmd, NULL, OLECMDID_SETPROGRESSPOS, OLECMDEXECOPT_DONTPROMPTUSER,
+                               &progress, NULL);
+    }
+
+    if(doc->usermode == EDITMODE && doc->hostui) {
+        DOCHOSTUIINFO hostinfo;
+
+        memset(&hostinfo, 0, sizeof(DOCHOSTUIINFO));
+        hostinfo.cbSize = sizeof(DOCHOSTUIINFO);
+        hres = IDocHostUIHandler_GetHostInfo(doc->hostui, &hostinfo);
+        if(SUCCEEDED(hres))
+            /* FIXME: use hostinfo */
+            TRACE("hostinfo = {%u %08x %08x %s %s}\n",
+                    hostinfo.cbSize, hostinfo.dwFlags, hostinfo.dwDoubleClick,
+                    debugstr_w(hostinfo.pchHostCss), debugstr_w(hostinfo.pchHostNS));
+    }
+}
+
+static void set_downloading_proc(task_t *_task)
+{
+    HTMLDocumentObj *doc = ((docobj_task_t*)_task)->doc;
+    IOleCommandTarget *olecmd;
+    HRESULT hres;
+
+    TRACE("(%p)\n", doc);
+
+    if(doc->frame)
+        IOleInPlaceFrame_SetStatusText(doc->frame, NULL /* FIXME */);
+
+    if(!doc->client)
+        return;
+
+    hres = IOleClientSite_QueryInterface(doc->client, &IID_IOleCommandTarget, (void**)&olecmd);
+    if(SUCCEEDED(hres)) {
+        VARIANT var;
+
+        V_VT(&var) = VT_I4;
+        V_I4(&var) = 1;
+
+        IOleCommandTarget_Exec(olecmd, NULL, OLECMDID_SETDOWNLOADSTATE, OLECMDEXECOPT_DONTPROMPTUSER,
+                               &var, NULL);
+        IOleCommandTarget_Release(olecmd);
+    }
+
+    if(doc->hostui) {
+        IDropTarget *drop_target = NULL;
+
+        hres = IDocHostUIHandler_GetDropTarget(doc->hostui, NULL /* FIXME */, &drop_target);
+        if(drop_target) {
+            FIXME("Use IDropTarget\n");
+            IDropTarget_Release(drop_target);
+        }
+    }
+}
+
+static HRESULT set_moniker(HTMLDocument *This, IMoniker *mon, IBindCtx *pibc)
 {
     nsChannelBSC *bscallback;
     LPOLESTR url = NULL;
-    task_t *task;
+    docobj_task_t *task;
     HRESULT hres;
     nsresult nsres;
 
@@ -113,8 +191,7 @@ static HRESULT set_moniker(HTMLDocument *This, IMoniker *mon, IBindCtx *pibc, BO
         }
     }
 
-    This->doc_obj->readystate = READYSTATE_LOADING;
-    call_property_onchanged(&This->cp_propnotif, DISPID_READYSTATE);
+    set_ready_state(This->window, READYSTATE_LOADING);
     update_doc(This, UPDATE_TITLE);
 
     HTMLDocument_LockContainer(This->doc_obj, TRUE);
@@ -127,7 +204,7 @@ static HRESULT set_moniker(HTMLDocument *This, IMoniker *mon, IBindCtx *pibc, BO
 
     TRACE("got url: %s\n", debugstr_w(url));
 
-    set_current_mon(This, mon);
+    set_current_mon(This->window, mon);
 
     if(This->doc_obj->client) {
         VARIANT silent, offline;
@@ -166,22 +243,14 @@ static HRESULT set_moniker(HTMLDocument *This, IMoniker *mon, IBindCtx *pibc, BO
     bscallback = create_channelbsc(mon);
 
     if(This->doc_obj->frame) {
-        task = heap_alloc(sizeof(task_t));
-
-        task->doc = This;
-        task->task_id = TASK_SETPROGRESS;
-        task->next = NULL;
-
-        push_task(task);
+        task = heap_alloc(sizeof(docobj_task_t));
+        task->doc = This->doc_obj;
+        push_task(&task->header, set_progress_proc, This->doc_obj->basedoc.task_magic);
     }
 
-    task = heap_alloc(sizeof(task_t));
-
-    task->doc = This;
-    task->task_id = TASK_SETDOWNLOADSTATE;
-    task->next = NULL;
-
-    push_task(task);
+    task = heap_alloc(sizeof(docobj_task_t));
+    task->doc = This->doc_obj;
+    push_task(&task->header, set_downloading_proc, This->doc_obj->basedoc.task_magic);
 
     if(This->doc_obj->nscontainer) {
         This->doc_obj->nscontainer->bscallback = bscallback;
@@ -196,15 +265,23 @@ static HRESULT set_moniker(HTMLDocument *This, IMoniker *mon, IBindCtx *pibc, BO
         }
     }
 
-    set_document_bscallback(This, bscallback);
+    set_window_bscallback(This->window, bscallback);
     IUnknown_Release((IUnknown*)bscallback);
     CoTaskMemFree(url);
 
-    if(bind_complete)
-        *bind_complete = FALSE;
     return S_OK;
 }
 
+void set_ready_state(HTMLWindow *window, READYSTATE readystate)
+{
+    window->readystate = readystate;
+    if(window->doc_obj && window->doc_obj->basedoc.window == window)
+        call_property_onchanged(&window->doc_obj->basedoc.cp_propnotif, DISPID_READYSTATE);
+    if(window->frame_element)
+        fire_event(window->frame_element->element.node.doc, EVENTID_READYSTATECHANGE,
+                   window->frame_element->element.node.nsnode, NULL);
+}
+
 static HRESULT get_doc_string(HTMLDocumentNode *This, char **str)
 {
     nsIDOMNode *nsnode;
@@ -282,19 +359,15 @@ static HRESULT WINAPI PersistMoniker_Load(IPersistMoniker *iface, BOOL fFullyAva
         IMoniker *pimkName, LPBC pibc, DWORD grfMode)
 {
     HTMLDocument *This = PERSISTMON_THIS(iface);
-    BOOL bind_complete = FALSE;
     HRESULT hres;
 
     TRACE("(%p)->(%x %p %p %08x)\n", This, fFullyAvailable, pimkName, pibc, grfMode);
 
-    hres = set_moniker(This, pimkName, pibc, &bind_complete);
+    hres = set_moniker(This, pimkName, pibc);
     if(FAILED(hres))
         return hres;
 
-    if(!bind_complete)
-        return start_binding(This, (BSCallback*)This->doc_obj->bscallback, pibc);
-
-    return S_OK;
+    return start_binding(This->window, NULL, (BSCallback*)This->window->bscallback, pibc);
 }
 
 static HRESULT WINAPI PersistMoniker_Save(IPersistMoniker *iface, IMoniker *pimkName,
@@ -318,11 +391,11 @@ static HRESULT WINAPI PersistMoniker_GetCurMoniker(IPersistMoniker *iface, IMoni
 
     TRACE("(%p)->(%p)\n", This, ppimkName);
 
-    if(!This->doc_obj->mon)
+    if(!This->window || !This->window->mon)
         return E_UNEXPECTED;
 
-    IMoniker_AddRef(This->doc_obj->mon);
-    *ppimkName = This->doc_obj->mon;
+    IMoniker_AddRef(This->window->mon);
+    *ppimkName = This->window->mon;
     return S_OK;
 }
 
@@ -551,12 +624,12 @@ static HRESULT WINAPI PersistStreamInit_Load(IPersistStreamInit *iface, LPSTREAM
         return hres;
     }
 
-    hres = set_moniker(This, mon, NULL, NULL);
+    hres = set_moniker(This, mon, NULL);
     IMoniker_Release(mon);
     if(FAILED(hres))
         return hres;
 
-    return channelbsc_load_stream(This->doc_obj->bscallback, pStm);
+    return channelbsc_load_stream(This->window->bscallback, pStm);
 }
 
 static HRESULT WINAPI PersistStreamInit_Save(IPersistStreamInit *iface, LPSTREAM pStm,
