@@ -448,6 +448,15 @@ PVOID RtlEnumerateGenericTableWithoutSplaying(PVOID pTable, PVOID *RestartKey)
     return NULL;
 }
 
+/******************************************************************************
+ *  RtlNumberGenericTableElements           [NTDLL.@]
+ */
+ULONG RtlNumberGenericTableElements(PVOID pTable)
+{
+    FIXME("(%p) stub!\n", pTable);
+    return 0;
+}
+
 /******************************************************************************
  *  RtlMoveMemory   [NTDLL.@]
  *
