@@ -42,6 +42,9 @@ static NDR_SCONTEXT (WINAPI *pNDRSContextUnmarshall2)(RPC_BINDING_HANDLE, void*,
 static RPC_STATUS (WINAPI *pRpcServerRegisterIfEx)(RPC_IF_HANDLE,UUID*, RPC_MGR_EPV*, unsigned int,
                    unsigned int,RPC_IF_CALLBACK_FN*);
 
+/* type check statements generated in header file */
+fnprintf *p_printf = printf;
+
 static void InitFunctionPointers(void)
 {
     HMODULE hrpcrt4 = GetModuleHandleA("rpcrt4.dll");
@@ -571,6 +574,16 @@ s_get_filename(void)
     return (char *)__FILE__;
 }
 
+int s_echo_ranged_int(int n)
+{
+    return n;
+}
+
+void s_get_ranged_enum(renum_t *re)
+{
+    *re = RE3;
+}
+
 void
 s_context_handle_test(void)
 {
@@ -703,6 +716,21 @@ s_get_numbers_struct(numbers_struct_t **ns)
     *(*ns)->numbers[0].pi = 5;
 }
 
+void
+s_full_pointer_test(int *a, int *b)
+{
+    ok(*a == 42, "Expected *a to be 42 instead of %d\n", *a);
+    ok(*b == 42, "Expected *b to be 42 instead of %d\n", *a);
+    ok(a == b, "Expected a (%p) to point to the same memory as b (%p)\n", a, b);
+}
+
+void
+s_full_pointer_null_test(int *a, int *b)
+{
+    ok(*a == 42, "Expected *a to be 42 instead of %d\n", *a);
+    ok(b == NULL, "Expected b to be NULL instead of %p\n", b);
+}
+
 void
 s_stop(void)
 {
@@ -762,6 +790,7 @@ basic_tests(void)
   wstr_struct_t ws = {wstring};
   str_t str;
   se_t se;
+  renum_t re;
 
   ok(int_return() == INT_CODE, "RPC int_return\n");
 
@@ -869,6 +898,17 @@ basic_tests(void)
   str = get_filename();
   ok(!strcmp(str, __FILE__), "get_filename() returned %s instead of %s\n", str, __FILE__);
   midl_user_free(str);
+
+  x = echo_ranged_int(0);
+  ok(x == 0, "echo_ranged_int() returned %d instead of 0\n", x);
+  x = echo_ranged_int(100);
+  ok(x == 100, "echo_ranged_int() returned %d instead of 100\n", x);
+
+  if (!old_windows_version)
+  {
+      get_ranged_enum(&re);
+      ok(re == RE3, "get_ranged_enum() returned %d instead of RE3\n", re);
+  }
 }
 
 static void
@@ -1055,6 +1095,7 @@ pointer_tests(void)
   void *buffer;
   int *pa2;
   s123_t *s123;
+  int val = 42;
 
   ok(test_list_length(list) == 3, "RPC test_list_length\n");
   ok(square_puint(p1) == 121, "RPC square_puint\n");
@@ -1115,6 +1156,9 @@ pointer_tests(void)
   s123 = get_s123();
   ok(s123->f1 == 1 && s123->f2 == 2 && s123->f3 == 3, "RPC get_s123\n");
   MIDL_user_free(s123);
+
+  full_pointer_test(&val, &val);
+  full_pointer_null_test(&val, NULL);
 }
 
 static int
@@ -1291,6 +1335,20 @@ client(const char *test)
     ok(RPC_S_OK == RpcStringFree(&binding), "RpcStringFree\n");
     ok(RPC_S_OK == RpcBindingFree(&IServer_IfHandle), "RpcBindingFree\n");
   }
+  else if (strcmp(test, "ncalrpc_basic") == 0)
+  {
+    static unsigned char ncalrpc[] = "ncalrpc";
+    static unsigned char guid[] = "00000000-4114-0704-2301-000000000000";
+    unsigned char *binding;
+
+    ok(RPC_S_OK == RpcStringBindingCompose(NULL, ncalrpc, NULL, guid, NULL, &binding), "RpcStringBindingCompose\n");
+    ok(RPC_S_OK == RpcBindingFromStringBinding(binding, &IServer_IfHandle), "RpcBindingFromStringBinding\n");
+
+    run_tests();
+
+    ok(RPC_S_OK == RpcStringFree(&binding), "RpcStringFree\n");
+    ok(RPC_S_OK == RpcBindingFree(&IServer_IfHandle), "RpcBindingFree\n");
+  }
   else if (strcmp(test, "np_basic") == 0)
   {
     static unsigned char np[] = "ncacn_np";
@@ -1316,11 +1374,17 @@ server(void)
   static unsigned char port[] = PORT;
   static unsigned char np[] = "ncacn_np";
   static unsigned char pipe[] = PIPE;
-  RPC_STATUS status, iptcp_status, np_status;
+  static unsigned char ncalrpc[] = "ncalrpc";
+  static unsigned char guid[] = "00000000-4114-0704-2301-000000000000";
+  RPC_STATUS status, iptcp_status, np_status, ncalrpc_status;
   DWORD ret;
 
   iptcp_status = RpcServerUseProtseqEp(iptcp, 20, port, NULL);
   ok(iptcp_status == RPC_S_OK, "RpcServerUseProtseqEp(ncacn_ip_tcp) failed with status %d\n", iptcp_status);
+
+  ncalrpc_status = RpcServerUseProtseqEp(ncalrpc, 0, guid, NULL);
+  ok(ncalrpc_status == RPC_S_OK, "RpcServerUseProtseqEp(ncalrpc) failed with status %d\n", ncalrpc_status);
+
   np_status = RpcServerUseProtseqEp(np, 0, pipe, NULL);
   if (np_status == RPC_S_PROTSEQ_NOT_SUPPORTED)
     skip("Protocol sequence ncacn_np is not supported\n");
@@ -1347,6 +1411,11 @@ server(void)
   else
     skip("tcp_basic tests skipped due to earlier failure\n");
 
+  if (ncalrpc_status == RPC_S_OK)
+    run_client("ncalrpc_basic");
+  else
+    skip("ncalrpc_basic tests skipped due to earlier failure\n");
+
   if (np_status == RPC_S_OK)
     run_client("np_basic");
   else
