@@ -883,6 +883,8 @@ static ALG_ID schannel_get_cipher_algid(gnutls_cipher_algorithm_t cipher)
     {
     case GNUTLS_CIPHER_UNKNOWN:
     case GNUTLS_CIPHER_NULL: return 0;
+    case GNUTLS_CIPHER_ARCFOUR_40:
+    case GNUTLS_CIPHER_ARCFOUR_128: return CALG_RC4;
     case GNUTLS_CIPHER_DES_CBC:
     case GNUTLS_CIPHER_3DES_CBC: return CALG_DES;
     case GNUTLS_CIPHER_AES_128_CBC:
@@ -958,8 +960,7 @@ static SECURITY_STATUS SEC_ENTRY schan_QueryContextAttributesW(
         case SECPKG_ATTR_REMOTE_CERT_CONTEXT:
         {
             unsigned int list_size;
-            const gnutls_datum_t *datum = pgnutls_certificate_get_peers(
-                    ctx->session, &list_size);
+            const gnutls_datum_t *datum;
 
             datum = pgnutls_certificate_get_peers(ctx->session, &list_size);
             if (datum)
@@ -1202,6 +1203,9 @@ static SECURITY_STATUS SEC_ENTRY schan_DecryptMessage(PCtxtHandle context_handle
                 return SEC_E_INTERNAL_ERROR;
             }
         }
+        else if (!ret)
+            break;
+
         received += ret;
     }
 
@@ -1385,7 +1389,7 @@ void SECUR32_initSchannelSP(void)
         pgnutls_global_set_log_function(schan_gnutls_log);
     }
 
-    schan_handle_table = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, 64 * sizeof(*schan_handle_table));
+    schan_handle_table = HeapAlloc(GetProcessHeap(), 0, 64 * sizeof(*schan_handle_table));
     if (!schan_handle_table)
     {
         ERR("Failed to allocate schannel handle table.\n");
@@ -1414,8 +1418,33 @@ fail:
 
 void SECUR32_deinitSchannelSP(void)
 {
+    SIZE_T i = schan_handle_count;
+
     if (!libgnutls_handle) return;
 
+    /* deinitialized sessions first because a pointer to the credentials
+     * are stored for the session by calling gnutls_credentials_set. */
+    while (i--)
+    {
+        if (schan_handle_table[i].type == SCHAN_HANDLE_CTX)
+        {
+            struct schan_context *ctx = schan_free_handle(i, SCHAN_HANDLE_CTX);
+            pgnutls_deinit(ctx->session);
+            HeapFree(GetProcessHeap(), 0, ctx);
+        }
+    }
+    i = schan_handle_count;
+    while (i--)
+    {
+        if (schan_handle_table[i].type != SCHAN_HANDLE_FREE)
+        {
+            struct schan_credentials *cred;
+            cred = schan_free_handle(i, SCHAN_HANDLE_CRED);
+            pgnutls_certificate_free_credentials(cred->credentials);
+            HeapFree(GetProcessHeap(), 0, cred);
+        }
+    }
+    HeapFree(GetProcessHeap(), 0, schan_handle_table);
     pgnutls_global_deinit();
     wine_dlclose(libgnutls_handle, NULL, 0);
 }
