@@ -27,13 +27,12 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d_texture);
 
-#define GLINFO_LOCATION (*gl_info)
-
 static void cubetexture_internal_preload(IWineD3DBaseTexture *iface, enum WINED3DSRGB srgb)
 {
     /* Override the IWineD3DResource Preload method. */
     IWineD3DCubeTextureImpl *This = (IWineD3DCubeTextureImpl *)iface;
     IWineD3DDeviceImpl *device = This->resource.wineD3DDevice;
+    struct wined3d_context *context = NULL;
     unsigned int i, j;
     BOOL srgb_mode;
     BOOL *dirty;
@@ -65,9 +64,9 @@ static void cubetexture_internal_preload(IWineD3DBaseTexture *iface, enum WINED3
      * activated at the beginning of drawPrimitive. */
     if (!device->isInDraw)
     {
-        /* No danger of recursive calls, ActivateContext sets isInDraw to true
+        /* No danger of recursive calls, context_acquire() sets isInDraw to true
          * when loading offscreen render targets into their texture. */
-        ActivateContext(device, NULL, CTXUSAGE_RESOURCELOAD);
+        context = context_acquire(device, NULL, CTXUSAGE_RESOURCELOAD);
     }
 
     if (This->resource.format_desc->format == WINED3DFMT_P8_UINT
@@ -109,6 +108,8 @@ static void cubetexture_internal_preload(IWineD3DBaseTexture *iface, enum WINED3
 
     /* No longer dirty. */
     *dirty = FALSE;
+
+    if (context) context_release(context);
 }
 
 static void cubetexture_cleanup(IWineD3DCubeTextureImpl *This)
@@ -138,14 +139,10 @@ static void cubetexture_cleanup(IWineD3DCubeTextureImpl *This)
     basetexture_cleanup((IWineD3DBaseTexture *)This);
 }
 
-#undef GLINFO_LOCATION
-
 /* *******************************************
    IWineD3DCubeTexture IUnknown parts follow
    ******************************************* */
 
-#define GLINFO_LOCATION This->resource.wineD3DDevice->adapter->gl_info
-
 static HRESULT WINAPI IWineD3DCubeTextureImpl_QueryInterface(IWineD3DCubeTexture *iface, REFIID riid, LPVOID *ppobj)
 {
     IWineD3DCubeTextureImpl *This = (IWineD3DCubeTextureImpl *)iface;
