@@ -26,7 +26,6 @@
 #include "wined3d_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d);
-#define GLINFO_LOCATION This->wineD3DDevice->adapter->gl_info
 
 static const DWORD pixel_states_render[] =
 {
@@ -194,31 +193,31 @@ static const DWORD vertex_states_sampler[] =
  */
 static HRESULT stateblock_allocate_shader_constants(IWineD3DStateBlockImpl *object)
 {
-    const struct wined3d_gl_info *gl_info = &object->wineD3DDevice->adapter->gl_info;
+    IWineD3DDeviceImpl *device = object->wineD3DDevice;
 
     /* Allocate space for floating point constants */
     object->pixelShaderConstantF = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
-            sizeof(float) * gl_info->max_pshader_constantsF * 4);
+            sizeof(float) * device->d3d_pshader_constantF * 4);
     if (!object->pixelShaderConstantF) goto fail;
 
     object->changed.pixelShaderConstantsF = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
-            sizeof(BOOL) * gl_info->max_pshader_constantsF);
+            sizeof(BOOL) * device->d3d_pshader_constantF);
     if (!object->changed.pixelShaderConstantsF) goto fail;
 
     object->vertexShaderConstantF = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
-            sizeof(float) * gl_info->max_vshader_constantsF * 4);
+            sizeof(float) * device->d3d_vshader_constantF * 4);
     if (!object->vertexShaderConstantF) goto fail;
 
     object->changed.vertexShaderConstantsF = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
-            sizeof(BOOL) * gl_info->max_vshader_constantsF);
+            sizeof(BOOL) * device->d3d_vshader_constantF);
     if (!object->changed.vertexShaderConstantsF) goto fail;
 
     object->contained_vs_consts_f = HeapAlloc(GetProcessHeap(), 0,
-            sizeof(DWORD) * gl_info->max_vshader_constantsF);
+            sizeof(DWORD) * device->d3d_vshader_constantF);
     if (!object->contained_vs_consts_f) goto fail;
 
     object->contained_ps_consts_f = HeapAlloc(GetProcessHeap(), 0,
-            sizeof(DWORD) * gl_info->max_pshader_constantsF);
+            sizeof(DWORD) * device->d3d_pshader_constantF);
     if (!object->contained_ps_consts_f) goto fail;
 
     return WINED3D_OK;
@@ -242,7 +241,7 @@ static inline void stateblock_set_bits(DWORD *map, UINT map_size)
 }
 
 /* Set all members of a stateblock savedstate to the given value */
-static void stateblock_savedstates_set_all(SAVEDSTATES *states, const struct wined3d_gl_info *gl_info)
+static void stateblock_savedstates_set_all(SAVEDSTATES *states, DWORD vs_consts, DWORD ps_consts)
 {
     unsigned int i;
 
@@ -271,11 +270,11 @@ static void stateblock_savedstates_set_all(SAVEDSTATES *states, const struct win
     states->vertexShaderConstantsI = 0xffff;
 
     /* Dynamically sized arrays */
-    memset(states->pixelShaderConstantsF, TRUE, sizeof(BOOL) * gl_info->max_pshader_constantsF);
-    memset(states->vertexShaderConstantsF, TRUE, sizeof(BOOL) * gl_info->max_vshader_constantsF);
+    memset(states->pixelShaderConstantsF, TRUE, sizeof(BOOL) * ps_consts);
+    memset(states->vertexShaderConstantsF, TRUE, sizeof(BOOL) * vs_consts);
 }
 
-static void stateblock_savedstates_set_pixel(SAVEDSTATES *states, const struct wined3d_gl_info *gl_info)
+static void stateblock_savedstates_set_pixel(SAVEDSTATES *states, const DWORD num_constants)
 {
     DWORD texture_mask = 0;
     WORD sampler_mask = 0;
@@ -298,10 +297,10 @@ static void stateblock_savedstates_set_pixel(SAVEDSTATES *states, const struct w
     states->pixelShaderConstantsB = 0xffff;
     states->pixelShaderConstantsI = 0xffff;
 
-    memset(states->pixelShaderConstantsF, TRUE, sizeof(BOOL) * gl_info->max_pshader_constantsF);
+    memset(states->pixelShaderConstantsF, TRUE, sizeof(BOOL) * num_constants);
 }
 
-static void stateblock_savedstates_set_vertex(SAVEDSTATES *states, const struct wined3d_gl_info *gl_info)
+static void stateblock_savedstates_set_vertex(SAVEDSTATES *states, const DWORD num_constants)
 {
     DWORD texture_mask = 0;
     WORD sampler_mask = 0;
@@ -325,12 +324,12 @@ static void stateblock_savedstates_set_vertex(SAVEDSTATES *states, const struct
     states->vertexShaderConstantsB = 0xffff;
     states->vertexShaderConstantsI = 0xffff;
 
-    memset(states->vertexShaderConstantsF, TRUE, sizeof(BOOL) * gl_info->max_vshader_constantsF);
+    memset(states->vertexShaderConstantsF, TRUE, sizeof(BOOL) * num_constants);
 }
 
 void stateblock_init_contained_states(IWineD3DStateBlockImpl *stateblock)
 {
-    const struct wined3d_gl_info *gl_info = &stateblock->wineD3DDevice->adapter->gl_info;
+    IWineD3DDeviceImpl *device = stateblock->wineD3DDevice;
     unsigned int i, j;
 
     for (i = 0; i <= WINEHIGHEST_RENDER_STATE >> 5; ++i)
@@ -357,7 +356,7 @@ void stateblock_init_contained_states(IWineD3DStateBlockImpl *stateblock)
         }
     }
 
-    for (i = 0; i < gl_info->max_vshader_constantsF; ++i)
+    for (i = 0; i < device->d3d_vshader_constantF; ++i)
     {
         if (stateblock->changed.vertexShaderConstantsF[i])
         {
@@ -384,7 +383,7 @@ void stateblock_init_contained_states(IWineD3DStateBlockImpl *stateblock)
         }
     }
 
-    for (i = 0; i < gl_info->max_pshader_constantsF; ++i)
+    for (i = 0; i < device->d3d_pshader_constantF; ++i)
     {
         if (stateblock->changed.pixelShaderConstantsF[i])
         {
@@ -536,13 +535,6 @@ static ULONG  WINAPI IWineD3DStateBlockImpl_Release(IWineD3DStateBlock *iface) {
 /**********************************************************
  * IWineD3DStateBlockImpl parts follows
  **********************************************************/
-static HRESULT  WINAPI IWineD3DStateBlockImpl_GetParent(IWineD3DStateBlock *iface, IUnknown **pParent) {
-    IWineD3DStateBlockImpl *This = (IWineD3DStateBlockImpl *)iface;
-    IUnknown_AddRef(This->parent);
-    *pParent = This->parent;
-    return WINED3D_OK;
-}
-
 static HRESULT  WINAPI IWineD3DStateBlockImpl_GetDevice(IWineD3DStateBlock *iface, IWineD3DDevice** ppDevice){
 
     IWineD3DStateBlockImpl *This   = (IWineD3DStateBlockImpl *)iface;
@@ -1162,7 +1154,7 @@ static HRESULT  WINAPI IWineD3DStateBlockImpl_InitStartupStateBlock(IWineD3DStat
     IWineD3DDevice_SetRenderState(device, WINED3DRS_SOFTWAREVERTEXPROCESSING, FALSE);
     tmpfloat.f = 1.0f;
     IWineD3DDevice_SetRenderState(device, WINED3DRS_POINTSIZE,                tmpfloat.d);
-    tmpfloat.f = ((IWineD3DImpl *)This->wineD3DDevice->wineD3D)->dxVersion < 9 ? 0.0f : 1.0f;
+    tmpfloat.f = 1.0f;
     IWineD3DDevice_SetRenderState(device, WINED3DRS_POINTSIZE_MIN,            tmpfloat.d);
     IWineD3DDevice_SetRenderState(device, WINED3DRS_POINTSPRITEENABLE,        FALSE);
     IWineD3DDevice_SetRenderState(device, WINED3DRS_POINTSCALEENABLE,         FALSE);
@@ -1178,7 +1170,7 @@ static HRESULT  WINAPI IWineD3DStateBlockImpl_InitStartupStateBlock(IWineD3DStat
     tmpfloat.f = 1.0f;
     IWineD3DDevice_SetRenderState(device, WINED3DRS_PATCHSEGMENTS,            tmpfloat.d);
     IWineD3DDevice_SetRenderState(device, WINED3DRS_DEBUGMONITORTOKEN,        0xbaadcafe);
-    tmpfloat.f = gl_info->max_pointsize;
+    tmpfloat.f = gl_info->limits.pointsize_max;
     IWineD3DDevice_SetRenderState(device, WINED3DRS_POINTSIZE_MAX,            tmpfloat.d);
     IWineD3DDevice_SetRenderState(device, WINED3DRS_INDEXEDVERTEXBLENDENABLE, FALSE);
     IWineD3DDevice_SetRenderState(device, WINED3DRS_COLORWRITEENABLE,         0x0000000F);
@@ -1272,7 +1264,7 @@ static HRESULT  WINAPI IWineD3DStateBlockImpl_InitStartupStateBlock(IWineD3DStat
         This->samplerState[i][WINED3DSAMP_DMAPOFFSET       ] = 0; /* TODO: Vertex offset in the presampled displacement map */
     }
 
-    for (i = 0; i < gl_info->max_textures; ++i)
+    for (i = 0; i < gl_info->limits.textures; ++i)
     {
         /* Note: This avoids calling SetTexture, so pretend it has been called */
         This->changed.textures |= 1 << i;
@@ -1329,23 +1321,19 @@ static const IWineD3DStateBlockVtbl IWineD3DStateBlock_Vtbl =
     IWineD3DStateBlockImpl_AddRef,
     IWineD3DStateBlockImpl_Release,
     /* IWineD3DStateBlock */
-    IWineD3DStateBlockImpl_GetParent,
     IWineD3DStateBlockImpl_GetDevice,
     IWineD3DStateBlockImpl_Capture,
     IWineD3DStateBlockImpl_Apply,
     IWineD3DStateBlockImpl_InitStartupStateBlock
 };
 
-HRESULT stateblock_init(IWineD3DStateBlockImpl *stateblock, IWineD3DDeviceImpl *device,
-        WINED3DSTATEBLOCKTYPE type, IUnknown *parent)
+HRESULT stateblock_init(IWineD3DStateBlockImpl *stateblock, IWineD3DDeviceImpl *device, WINED3DSTATEBLOCKTYPE type)
 {
-    const struct wined3d_gl_info *gl_info = &device->adapter->gl_info;
     unsigned int i;
     HRESULT hr;
 
     stateblock->lpVtbl = &IWineD3DStateBlock_Vtbl;
     stateblock->ref = 1;
-    stateblock->parent = parent;
     stateblock->wineD3DDevice = device;
     stateblock->blockType = type;
 
@@ -1368,16 +1356,17 @@ HRESULT stateblock_init(IWineD3DStateBlockImpl *stateblock, IWineD3DDeviceImpl *
     {
         case WINED3DSBT_ALL:
             stateblock_init_lights(stateblock, device->stateBlock->lightMap);
-            stateblock_savedstates_set_all(&stateblock->changed, gl_info);
+            stateblock_savedstates_set_all(&stateblock->changed, device->d3d_vshader_constantF,
+                                           device->d3d_pshader_constantF);
             break;
 
         case WINED3DSBT_PIXELSTATE:
-            stateblock_savedstates_set_pixel(&stateblock->changed, gl_info);
+            stateblock_savedstates_set_pixel(&stateblock->changed, device->d3d_pshader_constantF);
             break;
 
         case WINED3DSBT_VERTEXSTATE:
             stateblock_init_lights(stateblock, device->stateBlock->lightMap);
-            stateblock_savedstates_set_vertex(&stateblock->changed, gl_info);
+            stateblock_savedstates_set_vertex(&stateblock->changed, device->d3d_vshader_constantF);
             break;
 
         default:
