@@ -730,6 +730,26 @@ static char* heap_printf(const char *format, ...)
     return ret;
 }
 
+static void write_xml_text(FILE *file, const char *text)
+{
+    int i;
+    for (i = 0; text[i]; i++)
+    {
+        if (text[i] == '&')
+            fputs("&amp;", file);
+        else if (text[i] == '<')
+            fputs("&lt;", file);
+        else if (text[i] == '>')
+            fputs("&gt;", file);
+        else if (text[i] == '\'')
+            fputs("&apos;", file);
+        else if (text[i] == '"')
+            fputs("&quot;", file);
+        else
+            fputc(text[i], file);
+    }
+}
+
 static BOOL create_directories(char *directory)
 {
     BOOL ret = TRUE;
@@ -971,8 +991,12 @@ static BOOL write_menu_file(const char *unix_link, const char *filename)
             struct stat st;
             name[i] = 0;
             fprintf(tempfile, "  <Menu>\n");
-            fprintf(tempfile, "    <Name>%s%s</Name>\n", count ? "" : "wine-", name);
-            fprintf(tempfile, "    <Directory>%s%s.directory</Directory>\n", count ? "" : "wine-", name);
+            fprintf(tempfile, "    <Name>%s", count ? "" : "wine-");
+            write_xml_text(tempfile, name);
+            fprintf(tempfile, "</Name>\n");
+            fprintf(tempfile, "    <Directory>%s", count ? "" : "wine-");
+            write_xml_text(tempfile, name);
+            fprintf(tempfile, ".directory</Directory>\n");
             dir_file_name = heap_printf("%s/desktop-directories/%s%s.directory",
                 xdg_data_dir, count ? "" : "wine-", name);
             if (dir_file_name)
@@ -989,7 +1013,9 @@ static BOOL write_menu_file(const char *unix_link, const char *filename)
     name[i] = 0;
 
     fprintf(tempfile, "    <Include>\n");
-    fprintf(tempfile, "      <Filename>%s</Filename>\n", name);
+    fprintf(tempfile, "      <Filename>");
+    write_xml_text(tempfile, name);
+    fprintf(tempfile, "</Filename>\n");
     fprintf(tempfile, "    </Include>\n");
     for (i = 0; i < count; i++)
          fprintf(tempfile, "  </Menu>\n");
@@ -1805,10 +1831,18 @@ static BOOL write_freedesktop_mime_type_entry(const char *packages_dir, const ch
         {
             fprintf(packageFile, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
             fprintf(packageFile, "<mime-info xmlns=\"http://www.freedesktop.org/standards/shared-mime-info\">\n");
-            fprintf(packageFile, "  <mime-type type=\"%s\">\n", mime_type);
-            fprintf(packageFile, "    <glob pattern=\"*%s\"/>\n", dot_extension);
+            fprintf(packageFile, "  <mime-type type=\"");
+            write_xml_text(packageFile, mime_type);
+            fprintf(packageFile, "\">\n");
+            fprintf(packageFile, "    <glob pattern=\"*");
+            write_xml_text(packageFile, dot_extension);
+            fprintf(packageFile, "\"/>\n");
             if (comment)
-                fprintf(packageFile, "    <comment>%s</comment>\n", comment);
+            {
+                fprintf(packageFile, "    <comment>");
+                write_xml_text(packageFile, comment);
+                fprintf(packageFile, "</comment>\n");
+            }
             fprintf(packageFile, "  </mime-type>\n");
             fprintf(packageFile, "</mime-info>\n");
             ret = TRUE;
