@@ -50,6 +50,7 @@ struct wine_test
 char *tag = NULL;
 static struct wine_test *wine_tests;
 static int nr_of_files, nr_of_tests;
+static int nr_native_dlls;
 static const char whitespace[] = " \t\r\n";
 static const char testexe[] = "_test.exe";
 static char build_id[64];
@@ -174,21 +175,6 @@ static BOOL is_native_dll( HMODULE module )
     return TRUE;
 }
 
-/* check if Gecko is present, trying to trigger the install if not */
-static BOOL gecko_check(void)
-{
-    IHTMLDocument2 *doc;
-    IHTMLElement *body;
-    BOOL ret = FALSE;
-
-    CoInitialize( NULL );
-    if (FAILED( CoCreateInstance( &CLSID_HTMLDocument, NULL, CLSCTX_INPROC_SERVER,
-                                  &IID_IHTMLDocument2, (void **)&doc ))) return FALSE;
-    if ((ret = SUCCEEDED( IHTMLDocument2_get_body( doc, &body )))) IHTMLElement_Release( body );
-    IHTMLDocument_Release( doc );
-    return ret;
-}
-
 static void print_version (void)
 {
 #ifdef __i386__
@@ -208,6 +194,7 @@ static void print_version (void)
     const char *(CDECL *wine_get_build_id)(void);
     void (CDECL *wine_get_host_version)( const char **sysname, const char **release );
     BOOL (WINAPI *pIsWow64Process)(HANDLE hProcess, PBOOL Wow64Process);
+    BOOL (WINAPI *pGetProductInfo)(DWORD, DWORD, DWORD, DWORD, DWORD *);
 
     ver.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
     if (!(ext = GetVersionEx ((OSVERSIONINFO *) &ver)))
@@ -246,6 +233,15 @@ static void print_version (void)
              "    wSuiteMask=%d\n    wProductType=%d\n    wReserved=%d\n",
              ver.wServicePackMajor, ver.wServicePackMinor, ver.wSuiteMask,
              ver.wProductType, ver.wReserved);
+
+    pGetProductInfo = (void *)GetProcAddress(GetModuleHandleA("kernel32.dll"),"GetProductInfo");
+    if (pGetProductInfo && !running_under_wine())
+    {
+        DWORD prodtype = 0;
+
+        pGetProductInfo(ver.dwMajorVersion, ver.dwMinorVersion, ver.wServicePackMajor, ver.wServicePackMinor, &prodtype);
+        xprintf("    dwProductInfo=%u\n", prodtype);
+    }
 }
 
 static inline int is_dot_dir(const char* x)
@@ -679,12 +675,7 @@ extract_test_proc (HMODULE hModule, LPCTSTR lpszType,
     {
         FreeLibrary(dll);
         xprintf ("    %s=load error Configured as native\n", dllname);
-        return TRUE;
-    }
-    if (!strcmp( dllname, "mshtml" ) && running_under_wine() && !gecko_check())
-    {
-        FreeLibrary(dll);
-        xprintf ("    %s=load error Gecko is not installed\n", dllname);
+        nr_native_dlls++;
         return TRUE;
     }
     FreeLibrary(dll);
@@ -817,6 +808,9 @@ run_tests (char *logname, char *outdir)
 
     report (R_DELTA, 0, "Extracting: Done");
 
+    if (nr_native_dlls)
+        report( R_WARNING, "Some dlls are configured as native, you won't be able to submit results." );
+
     report (R_STATUS, "Running tests");
     report (R_PROGRESS, 1, nr_of_tests);
     for (i = 0; i < nr_of_files; i++) {
@@ -1054,7 +1048,7 @@ int main( int argc, char *argv[] )
 
         if (!logname) {
             logname = run_tests (NULL, outdir);
-            if (build_id[0] && !nb_filters &&
+            if (build_id[0] && !nb_filters && !nr_native_dlls &&
                 report (R_ASK, MB_YESNO, "Do you want to submit the test results?") == IDYES)
                 if (!send_file (logname) && !DeleteFileA(logname))
                     report (R_WARNING, "Can't remove logfile: %u", GetLastError());
